/*
 * Licensed Materials - Property of IBM
 *
 * Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA DP Schedule Contract with IBM Corp.
 */

egl.convertTextToStringN = function( str, n )
{
	if ( str != null && str.length > n )
	{
		return str.substring( 0, n );
	}
	return str;
};

egl.convertTextToFixedText = function( str, len )
{
	if ( str == null || str.length == len )
	{
		return str;
	}
	else if ( str.length > len )
	{
		return str.substring( 0, len );
	}
	else
	{
		return str + egl.egl.core.$StrLib.spaces( len - str.length );
	}
};

egl.padWithLeadingZeros = function(s, totChars)
{
	var zerosToAdd = (totChars - egl.egl.core.$StrLib.textLen(s));
	{
		var i = 1;
		for ( ; (i <= zerosToAdd); i = (i + 1) )
		{
			s = ("0" + s);
		}
	}
	return s;
};

egl.getDateInfo = function( d )
{
	var day = egl.padWithLeadingZeros("" + d.getDate(),2);
	var month = egl.padWithLeadingZeros("" + (d.getMonth()+1),2);
	var year = egl.padWithLeadingZeros("" + Math.min( 9999,d.getFullYear() ), 4 );
	var twoDigitYear = year.substr(2,2);
	
	var result = new Array();
	result[0] = year;
	result[1] = twoDigitYear;
	result[2] = month;
	result[3] = day;
	return result;
};

egl.dateToString = function( d, format )
{
	var dateInfo = egl.getDateInfo(d);
	var result = "";
	
	var validCheck = format.replace(/yyyy|yy|MM|dd|''|'([^']|'')*'|[^A-Za-z]/g, "");
	if(validCheck.length > 0)
	{
		throw egl.createTypeCastException( "CRRUI2717E", [format] );
	}
	
	var tokens = format.match(/yyyy|yy|MM|dd|''|'([^']|'')*'|[^A-Za-z]/g);
	var numTokens = (tokens == null) ? 0 : tokens.length;
	for(var i = 0; i < numTokens; i++ )
	{
		if(tokens[i] == "yyyy")
		{
			result += dateInfo[0];
		}
		else if(tokens[i] == "yy")
		{
			result += dateInfo[1];
		}
		else if(tokens[i] == "MM")
		{
			result += dateInfo[2];
		}
		else if(tokens[i] == "dd")
		{
			result += dateInfo[3];
		}
		else if(tokens[i] == "''")
		{
			result += "'";
		}
		else if(tokens[i].charAt(0) == "'")
		{
			var lit = tokens[i].substr(1,tokens[i].length-2);
			result += lit.replace(/''/, "'");
		}
		else
		{
			result += tokens[i];
		}
	}
	
	return result;
};

egl.getTimeInfo = function( t )
{
	var hours24 = t.getHours();
	var hours12;
	var minutes = egl.padWithLeadingZeros("" + t.getMinutes(),2);
	var seconds = egl.padWithLeadingZeros("" + t.getSeconds(),2);
	var amPm;

	if( hours24 <= 11 )
	{
		amPm = "AM";
		if( hours24 == 0 )
		{
			hours12 = 12;
		}
		else
		{
			hours12 = hours24;
		}
	}
	else
	{
		amPm = "PM";
		if( hours24 != 12 )
		{
			hours12 = (hours24 - 12);
		}
		else
		{
			hours12 = 12;
		}
	}
	
	hours24 = egl.padWithLeadingZeros("" + hours24,2);
	hours12 = egl.padWithLeadingZeros("" + hours12,2);
	
	var result = new Array();
	result[0] = hours24;
	result[1] = hours12;
	result[2] = minutes;
	result[3] = seconds;
	result[4] = amPm;
	return result;
};

egl.timeToString = function( t, format )
{
	var timeParts = egl.getTimeInfo(t);
	var result = "";
	
	var validCheck = format.replace(/HH|hh|mm|ss|a|''|'([^']|'')*'|[^A-Za-z]/g, "");
	if(validCheck.length > 0)
	{
		throw egl.createTypeCastException( "CRRUI2717E", [format] );
	}
	
	var tokens = format.match(/HH|hh|mm|ss|a|''|'([^']|'')*'|[^A-Za-z]/g);
	var numTokens = (tokens == null) ? 0 : tokens.length;
	for(var i = 0; i < numTokens; i++ )
	{
		if(tokens[i] == "HH")
		{
			result += timeParts[0];
		}
		else if(tokens[i] == "hh")
		{
			result += timeParts[1];
		}
		else if(tokens[i] == "mm")
		{
			result += timeParts[2];
		}
		else if(tokens[i] == "ss")
		{
			result += timeParts[3];
		}
		else if(tokens[i] == "a")
		{
			result += timeParts[4];
		}
		else if(tokens[i] == "''")
		{
			result += "'";
		}
		else if(tokens[i].charAt(0) == "'")
		{
			var lit = tokens[i].substr(1,tokens[i].length-2);
			result += lit.replace(/''/, "'");
		}
		else
		{
			result += tokens[i];
		}
	}
	
	return result;
};

egl.stringToTime = function(s, format)
{
	var result = egl.stringToTimeInternal(s, format, true);
	if( result == null )
	{
		result = egl.stringToTimeInternal(s, format, false);
	}
	if( result == null && s.length < format.length )
	{
		result = egl.stringToTimeInternal("0" + s, format, false);
	}
	
	if( result != null )
	{
		return result;
	}
	else
	{
		throw egl.createTypeCastException( "CRRUI2017E", [ s, "string", "time" ], "time", "string" );
	}
};

egl.stringToTimeInternal = function(s, format, strict)
{
	var result = new Date;
	var tokens = format.match(/HH|hh|mm|ss|a|''|'([^']|'')*'|[^A-Za-z]/g);
	var numTokens = (tokens == null) ? 0 : tokens.length;
	result.setSeconds(0);
	
	if(!strict)
	{
		s = s.replace(/[^0-9]/g, "");
	}
	
	for(var i = 0; i < numTokens; i++ )
	{
		s = egl.processToken(tokens[i], s, result, strict);
		if(s == null)
		{
			return null;
		}
	}
	
	if("PM" == result["eze$$AMPM"] && result.getHours() < 12)
	{
		result.setHours(result.getHours() + 12);
	}
	else if("AM" == result["eze$$AMPM"] && result.getHours() == 12)
	{
		result.setHours(result.getHours() - 12);
	}
	
	return result;
};

egl.stringToDate = function(s, format)
{
	var result = egl.stringToDateInternal(s, format, true);
	if( result == null )
	{
		result = egl.stringToDateInternal(s, format, false);
	}
	if( result == null && s.length < format.length )
	{
		result = egl.stringToDateInternal("0" + s, format, false);
	}
	
	if( result != null )
	{
		result.setHours(0);
		result.setMinutes(0);
		result.setSeconds(0);
		result.setMilliseconds(0);
		return result;
	}
	else
	{
		throw egl.createTypeCastException( "CRRUI2017E", [ s, "string", "date" ], "date", "string" );
	}
};

egl.stringToDateInternal = function(s, format, strict)
{
	var result = new Date(1970, 0, 1);
	var tokens = format.match(/yyyy|yy|MM|dd|''|'([^']|'')*'|[^A-Za-z]/g);
	var numTokens = (tokens == null) ? 0 : tokens.length;
	var adjustDate = false;
	
	if(!strict)
	{
		s = s.replace(/[^0-9]/g, "");
	}
	
	for(var i = 0; i < numTokens; i++ )
	{
		s = egl.processToken(tokens[i], s, result, strict);
		if(s == null)
		{
			return null;
		}
	}
	
	if (strict)
	{
		if (s != null && s.length > 0)
		{
			return null;
		}
	}
	
	if(result["eze$$adjustdate"])
	{
		result = egl.adjustYear(result);
	}
	
	return result;
};

egl.adjustYear = function(origDate)
{
	var d = (new Date);
	var curFullYear = d.getFullYear();
	var curTwoDigitPrefix = ("" + curFullYear).substr(0,2);

	var range = new Array();
	range[0] = new Date(d);
	range[0].setFullYear(range[0].getFullYear()-80);
	range[1] = new Date(d);
	range[1].setFullYear(range[1].getFullYear()+20);
	
	var newDate = new Date();
	newDate.setTime(origDate.getTime());
	var origTwoDigitYear = ("" + origDate.getFullYear());
	origTwoDigitYear = egl.padWithLeadingZeros(origTwoDigitYear.substr(origTwoDigitYear.length - 2),2);
	newDate.setFullYear("" + (curTwoDigitPrefix-1) + origTwoDigitYear);
	
	if( newDate.getTime() >= range[0].getTime() && newDate.getTime() <= range[1].getTime() )
	{
		return newDate;
	}
	
	newDate.setFullYear("" + curTwoDigitPrefix + origTwoDigitYear);
	if( newDate.getTime() >= range[0].getTime() && newDate.getTime() <= range[1].getTime() )
	{
		return newDate;
	}
	
	newDate.setFullYear("" + (curTwoDigitPrefix+1) + origTwoDigitYear);
	return newDate;
};

egl.processToken = function(token, s, result, strict)
{
	if(token == "yyyy")
	{
		if(s.length < 4)
		{
			return null;
		}
		
		var sub = s.substr(0,4);
		if( egl.isnumeric(sub) )
		{
			result.setFullYear(sub);
			s = s.substr(4);
		}
		else
		{
			return null;
		}
	}
	else if(token == "yy")
	{
		if(s.length < 2)
		{
			return null;
		}

		result["eze$$adjustdate"] = true;
		var sub = s.substr(0,2);
		if( egl.isnumeric(sub) )
		{
			result.setFullYear(sub);
			s = s.substr(2);
		}
		else
		{
			return null;
		}
	}
	else if(token == "MM")
	{
		if(s.length < 2)
		{
			return null;
		}

		var sub = s.substr(0,2);
		if( egl.isnumeric(sub) && sub >= 1 && sub <= 12)
		{
			result.setMonth(sub-1);
			s = s.substr(2);
		}
		else
		{
			return null;
		}
	}
	else if(token == "dd")
	{
		if(s.length < 2)
		{
			return null;
		}

		var sub = s.substr(0,2);
		if( egl.isnumeric(sub) && sub >= 1 && sub <= 31 )
		{
			result.setDate(sub);
			s = s.substr(2);
		}
		else
		{
			return null;
		}
	}
	else if(token == "HH")
	{
		if(s.length < 2)
		{
			return null;
		}
		
		var sub = s.substr(0,2);
		if( egl.isnumeric(sub) && sub >= 0 && sub <= 23 )
		{
			result.setHours(sub);
			s = s.substr(2);
		}
		else
		{
			return null;
		}
	}
	else if(token == "hh")
	{
		if(s.length < 2)
		{
			return null;
		}
		
		var sub = s.substr(0,2);
		if( egl.isnumeric(sub) && sub >= 1 && sub <= 12 )
		{
			result.setHours(sub);
			s = s.substr(2);
		}
		else
		{
			return null;
		}
	}
	else if(token == "mm")
	{
		if(s.length < 2)
		{
			return null;
		}
		
		var sub = s.substr(0,2);
		if( egl.isnumeric(sub) && sub >= 0 && sub <= 59 )
		{
			result.setMinutes(sub);
			s = s.substr(2);
		}
		else
		{
			return null;
		}
	}
	else if(token == "ss")
	{
		if(s.length < 2)
		{
			return null;
		}
		
		var sub = s.substr(0,2);
		if( egl.isnumeric(sub) && sub >= 0 && sub <= 59 )
		{
			result.setSeconds(sub);
			s = s.substr(2);
		}
		else
		{
			return null;
		}
	}
	else if(token == "SSSSSS")
	{
		if(s.length < 6)
		{
			return null;
		}
		
		result.setMilliseconds(s.substr(0,3));
		s = s.substr(6);
	}
	else if(token == "a")
	{
		var amPm = s.substr(0,2).toUpperCase();
		if(amPm == "PM")
		{
			result["eze$$AMPM"] = "PM";
		}
		else if(amPm == "AM")
		{
			result["eze$$AMPM"] = "AM";
		}
		else
		{
			return null;
		}
		s = s.substr(2);
	}
	else if( strict )
	{
		if(token == "''")
		{
			if(s.charAt(0) != "'")
			{
				return null;				
			}
			s = s.substr(1);
		}
		else if(token.charAt(0) == "'")
		{
			var lit = token.substr(1,token.length-2);
			lit = lit.replace(/''/, "'");
			if(s.substr(0, lit.length) != lit)
			{
				return null;
			}
			s = s.substr(lit.length);
		}
		else
		{
			if(s.substr(0, token.length) != token)
			{
				return null;
			}
			s = s.substr(token.length);
		}
	}
	
	return s;
};

egl.timeStampToString = function( ts, format )
{
	var dateParts = egl.getDateInfo(ts);
	var timeParts = egl.getTimeInfo(ts);
	var result = "";
	
	var validCheck = format.replace(/yyyy|yy|MM|dd|HH|hh|mm|ss|SSSSSS|a|''|'([^']|'')*'|[^A-Za-z]/g, "");
	if(validCheck.length > 0)
	{
		throw egl.createTypeCastException( "CRRUI2717E", [format] );
	}
	
	var tokens = format.match(/yyyy|yy|MM|dd|HH|hh|mm|ss|SSSSSS|a|''|'([^']|'')*'|[^A-Za-z]/g);
	var numTokens = (tokens == null) ? 0 : tokens.length;
	for(var i = 0; i < numTokens; i++ )
	{
		if(tokens[i] == "yyyy")
		{
			result += dateParts[0];
		}
		else if(tokens[i] == "yy")
		{
			result += dateParts[1];
		}
		else if(tokens[i] == "MM")
		{
			result += dateParts[2];
		}
		else if(tokens[i] == "dd")
		{
			result += dateParts[3];
		}
		else if(tokens[i] == "HH")
		{
			result += timeParts[0];
		}
		else if(tokens[i] == "hh")
		{
			result += timeParts[1];
		}
		else if(tokens[i] == "mm")
		{
			result += timeParts[2];
		}
		else if(tokens[i] == "ss")
		{
			result += timeParts[3];
		}
		else if(tokens[i] == "a")
		{
			result += timeParts[4];
		}
		else if(tokens[i] == "SSSSSS")
		{
			result += (egl.padWithLeadingZeros("" + ts.getMilliseconds(),3) + "000");
		}
		else if(tokens[i] == "''")
		{
			result += "'";
		}
		else if(tokens[i].charAt(0) == "'")
		{
			var lit = tokens[i].substr(1,tokens[i].length-2);
			result += lit.replace(/''/, "'");
		}
		else
		{
			result += tokens[i];
		}
	}
	
	return result;
};

egl.stringToTimeStamp = function(s, format)
{
	var result = egl.stringToTimeStampInternal(s, format, true);
	if( result == null )
	{
		result = egl.stringToTimeStampInternal(s, format, false);
	}
	if( result == null && s.length < format.length )
	{
		result = egl.stringToTimeStampInternal("0" + s, format, false);
	}
	
	if( result != null )
	{
		return result;
	}
	else
	{
		throw egl.createTypeCastException( "CRRUI2017E", [ s, "string", "timestamp" ], "time", "timestamp" );
	}
};

egl.stringToTimeStampInternal = function( s, format, strict )
{
	var result = new Date;
	result.setFullYear(0);
	result.setMonth(0);
	result.setDate(1);
	result.setHours(0);
	result.setMinutes(0);
	result.setSeconds(0);
	result.setMilliseconds(0);

	var tokens = format.match(/yyyy|yy|MM|dd|HH|hh|mm|ss|SSSSSS|a|''|'([^']|'')*'|[^A-Za-z]/g);
	var numTokens = (tokens == null) ? 0 : tokens.length;
	
	if(!strict)
	{
		s = s.replace(/[^0-9]/g, "");
	}
	
	for(var i = 0; i < numTokens; i++ )
	{
		s = egl.processToken(tokens[i], s, result, strict);
		if(s == null)
		{
			return null;
		}
	}
	
	if(result["eze$$adjustdate"])
	{
		result = egl.adjustYear(result);
	}
	if("PM" == result["eze$$AMPM"] && result.getHours() < 12)
	{
		result.setHours(result.getHours() + 12);
	}
	else if("AM" == result["egl$AMPM"] && result.getHours() == 12)
	{
		result.setHours(result.getHours() - 12);
	}

	return result;
};

egl.convertNumberToInteger = function( x, upper, lower, creatx )
{
	if ( !isFinite( x ) )
	{
		var target = upper == 2147483647 ? "int" : "smallint";
		if (!creatx)
		{
			creatx = egl.createTypeCastException;
		}
		throw creatx( "CRRUI2018E", [ String( x ), target ] );
	}

	if ( x >= 0 )
	{
		x = Math.floor( x );
		if ( x > upper )
		{
			var target = upper == 2147483647 ? "int" : "smallint";
			if (!creatx)
			{
				creatx = egl.createTypeCastException;
			}
			throw creatx( "CRRUI2018E", [ String( x ), target ] );
		}
	}
	else
	{
		x = Math.ceil( x );
		if ( x < lower )
		{
			var target = upper == 2147483647 ? "int" : "smallint";
			if (!creatx)
			{
				creatx = egl.createTypeCastException;
			}
			throw creatx( "CRRUI2018E", [ String( x ), target ] );
		}
	}

	return x;
};

egl.convertFloatToInteger = function( x, upper, lower, creatx )
{
	if ( !isFinite( x ) )
	{
		var target = upper == 2147483647 ? "int" : "smallint";
		if (!creatx)
		{
			creatx = egl.createTypeCastException;
		}
		throw creatx( "CRRUI2018E", [ String( x ), target ] );
	}

	if ( x > 0 )
	{
		x = Math.round( x );
		if ( x > upper )
		{
			var target = upper == 2147483647 ? "int" : "smallint";
			if (!creatx)
			{
				creatx = egl.createTypeCastException;
			}
			throw creatx( "CRRUI2018E", [ String( x ), target ] );
		}
	}
	else if ( x < 0 )
	{
		x = -x;
		x = Math.round( x );
		x = -x;
		if ( x < lower )
		{
			var target = upper == 2147483647 ? "int" : "smallint";
			if (!creatx)
			{
				creatx = egl.createTypeCastException;
			}
			throw creatx( "CRRUI2018E", [ String( x ), target ] );
		}
	}

	return x;
};

egl.convertNumberToInt = function( x, creatx )
{
	return egl.convertNumberToInteger( x, 2147483647, -2147483648, creatx );
};

egl.convertFloatToInt = function( x, creatx )
{
	return egl.convertFloatToInteger( x, 2147483647, -2147483648, creatx );
};

egl.convertNumberToSmallint = function( x, creatx )
{
	return egl.convertNumberToInteger( x, 32767, -32768, creatx );
};

egl.convertFloatToSmallint = function( x, creatx )
{
	return egl.convertFloatToInteger( x, 32767, -32768, creatx );
};

egl.convertIntegerToDecimal = function( x, limit, creatx )
{
	return egl.convertDecimalToDecimal( new egl.javascript.BigDecimal( String( x ) ), 0, limit, creatx );
};

egl.convertFloatToDecimal = function( x, decimals, limit, creatx )
{
	if ( !isFinite( x ) )
	{
		if (!creatx)
		{
			creatx = egl.createTypeCastException;
		}
		throw creatx( "CRRUI2018E", [ String( x ), "decimal" ] );
	}

	x = new egl.javascript.BigDecimal( x.toFixed( decimals ) );// toFixed rounds half-up so give an extra decimal, let the next call round down
	return egl.convertDecimalToDecimal( x, decimals, limit, creatx );
};

egl.convertDecimalToDecimal = function( x, decimals, limit, creatx )
{
	if ( x.scale() > decimals )
	{
		x = x.setScale( decimals, egl.javascript.BigDecimal.prototype.ROUND_DOWN );
	}

	if ( x.compareTo( limit ) > 0 || x.compareTo( limit.negate() ) < 0 )
	{
		if (!creatx)
		{
			creatx = egl.createTypeCastException;
		}
		throw creatx( "CRRUI2018E", [ String( x ), "decimal" ] );
	}

	return x;
};

egl.MAX_INT_BD = new egl.javascript.BigDecimal("2147483647");
egl.MIN_INT_BD = new egl.javascript.BigDecimal("-2147483648");
egl.MAX_SMALLINT_BD = new egl.javascript.BigDecimal("32767");
egl.MIN_SMALLINT_BD = new egl.javascript.BigDecimal("-32768");
egl.MAX_BIGINT_BD = new egl.javascript.BigDecimal("9223372036854775807");
egl.MIN_BIGINT_BD = new egl.javascript.BigDecimal("-9223372036854775808");
egl.MAX_SMALLFLOAT = 3.4028234663852886e38;
egl.MIN_SMALLFLOAT = 1.401298464324817e-45;

egl.convertDecimalToInteger = function( x, upper, lower, creatx )
{
	if ( x.signum() >= 0 )
	{
		x = x.setScale( 0, egl.javascript.BigDecimal.prototype.ROUND_FLOOR );
		if ( x.compareTo( upper ) > 0 )
		{
			var target = 
				upper === egl.MAX_INT_BD ? "int" 
					: upper === egl.MAX_BIGINT_BD ? "bigint" 
						: "smallint";
			if (!creatx)
			{
				creatx = egl.createTypeCastException;
			}
			throw creatx( "CRRUI2018E", [ String( x ), target ] );
		}
	}
	else
	{
		x = x.setScale( 0, egl.javascript.BigDecimal.prototype.ROUND_CEILING );
		if ( x.compareTo( lower ) < 0 )
		{
			var target = 
				upper === egl.MAX_INT_BD ? "int" 
					: upper === egl.MAX_BIGINT_BD ? "bigint" 
						: "smallint";
			if (!creatx)
			{
				creatx = egl.createTypeCastException;
			}
			throw creatx( "CRRUI2018E", [ String( x ), target ] );
		}
	}

	return x;
};

egl.convertDecimalToInt = function( x, creatx )
{
	return Number( egl.convertDecimalToInteger( x, egl.MAX_INT_BD, egl.MIN_INT_BD, creatx ) );
};

egl.convertDecimalToSmallint = function( x, creatx )
{
	return Number( egl.convertDecimalToInteger( x, egl.MAX_SMALLINT_BD, egl.MIN_SMALLINT_BD, creatx ) );
};

egl.convertDecimalToBigint = function( x, creatx )
{
	return egl.convertDecimalToInteger( x, egl.MAX_BIGINT_BD, egl.MIN_BIGINT_BD, creatx );
};

egl.convertFloatToBigint = function( x, creatx )
{
	if ( !isFinite( x ) )
	{
		if (!creatx)
		{
			creatx = egl.createTypeCastException;
		}
		throw creatx( "CRRUI2018E", [ String( x ), "bigint" ] );
	}

	x = new egl.javascript.BigDecimal( x.toFixed( 0 ) );
	return egl.convertDecimalToInteger( x, egl.MAX_BIGINT_BD, egl.MIN_BIGINT_BD, creatx );
};

egl.convertFloatToSmallfloat = function( x, creatx )
{
	if ( !isFinite( x ) || Math.abs( x ) > egl.MAX_SMALLFLOAT || (x != 0 && Math.abs( x ) < egl.MIN_SMALLFLOAT) )
	{
		if (!creatx)
		{
			creatx = egl.createTypeCastException;
		}
		throw creatx( "CRRUI2018E", [ String( x ), "smallfloat" ] );
	}
	
	return x;
};

egl.convertStringToInt = function( /*string*/ str ) {
	if (str=="") str="0";
	var asNum = Number( str );
	var num = parseInt( asNum, 10 );
	if ( isNaN( num ) ) {
		throw egl.createTypeCastException( "CRRUI2017E", [ str, "string", "int" ] );
	}
	return egl.convertNumberToInt( num );
};

egl.convertStringToSmallint = function( /*string*/ str ) {
	if (str=="") str="0";
	var asNum = Number( str );
	var num = parseInt( asNum, 10 );
	if ( isNaN( num ) ) {
		throw egl.createTypeCastException( "CRRUI2017E", [ str, "string", "smallint" ] );
	}
	return egl.convertNumberToSmallint( num );
};

egl.convertStringToFloat = function( /*string*/ str ) {
	if (str=="") str="0";
	var num = Number( str );
	if ( isNaN( num ) ) {
		throw egl.createTypeCastException( "CRRUI2017E", [ str, "string", "float" ] );
	}
	return num;
};

egl.convertStringToDecimal = function( /*string*/ str, decimals, limit ) {
	if (str=="") str="0";
	try
	{
		var num = new egl.javascript.BigDecimal( str );
	}
	catch ( oops )
	{
		throw egl.createTypeCastException( "CRRUI2017E", [ str, "string", "decimal" ] );
	}
	return egl.convertDecimalToDecimal( num, decimals, limit );
};

egl.convertStringToBigint = function( str ) {
	if (str=="") str="0";
	try
	{
		var num = new egl.javascript.BigDecimal( str );
	}
	catch ( oops )
	{
		throw egl.createTypeCastException( "CRRUI2017E", [ str, "string", "bigint" ] );
	}
	return egl.convertDecimalToInteger( num, egl.MAX_BIGINT_BD, egl.MIN_BIGINT_BD );
};

egl.convertDateToInt = function( /*date*/ date ) {
	var refDate = new Date( 1899, 11, 31 );
	var days = ( date.getTime() - refDate.getTime() ) / egl.MS_PER_DAY;
	return egl.convertFloatToInt( days );
};

egl.nullableconcat = function(var1, var2) {
	if (var1 == null || var2 == null)
		return null;
	return egl.convertToString(var1) + egl.convertToString(var2);
};

egl.nullableadd = function(var1, var2) {
	if (var1 == null || var2 == null)
		return null;
	if (var1 instanceof egl.javascript.BigDecimal ||
		var2 instanceof egl.javascript.BigDecimal)
	{
		return new egl.javascript.BigDecimal(var1.toString()).add(new egl.javascript.BigDecimal(var2.toString()));
	}
	return Number(var1) + Number(var2);
};

egl.nullablesubtract = function(var1, var2) {
	if (var1 == null || var2 == null)
		return null;
	if (var1 instanceof egl.javascript.BigDecimal ||
		var2 instanceof egl.javascript.BigDecimal)
	{
		return new egl.javascript.BigDecimal(var1.toString()).subtract(new egl.javascript.BigDecimal(var2.toString()));
	}
	return Number(var1) - Number(var2);
};

egl.nullablemultiply = function(var1, var2) {
	if (var1 == null || var2 == null)
		return null;
	if (var1 instanceof egl.javascript.BigDecimal ||
		var2 instanceof egl.javascript.BigDecimal)
	{
		return new egl.javascript.BigDecimal(var1.toString()).multiply(new egl.javascript.BigDecimal(var2.toString()));
	}
	return Number(var1) * Number(var2);
};

egl.divide = function(var1, var2) {
	if (var2 == 0)
		throw egl.createRuntimeException("CRRUI2037E", null);
	return var1 / var2;
};

egl.nullabledivide = function(var1, var2) {
	if (var1 == null || var2 == null)
		return null;
	var isfloat1 = (typeof var1 === 'number' && Math.round(var1)!==var1);
	var isfloat2 = (typeof var2 === 'number' && Math.round(var2)!==var2);
	if (isfloat1 || isfloat2)
	{
		var rhs = Number(var2.toString());
		if (rhs == 0)
			throw egl.createRuntimeException("CRRUI2037E", null);
		return Number(var1.toString()) / rhs;
	}
	
	var dec1 = new egl.javascript.BigDecimal(var1.toString());
	var dec2 = new egl.javascript.BigDecimal(var2.toString());
	return dec1.divide(dec2,egl.javascript.BigDecimal.prototype.eglMC);
};

egl.remainder = function(var1, var2) {
	if (var2 == 0)
		throw egl.createRuntimeException("CRRUI2037E", null);
	return var1 % var2;
};

egl.nullableremainder = function(var1, var2) {
	if (var1 == null || var2 == null)
		return null;
	if (var1 instanceof egl.javascript.BigDecimal ||
		var2 instanceof egl.javascript.BigDecimal)
	{
		return new egl.javascript.BigDecimal(var1.toString()).remainder(new egl.javascript.BigDecimal(var2.toString()));
	}
	var rhs = Number(var2);
	if (rhs == 0)
		throw egl.createRuntimeException("CRRUI2037E", null);
	return Number(var1) % rhs;
};

egl.nullablepow = function(var1, var2, forceDecimal) {
	if (var1 == null || var2 == null)
		return null;
	if (forceDecimal || var1 instanceof egl.javascript.BigDecimal || var2 instanceof egl.javascript.BigDecimal)
	{
		return new egl.javascript.BigDecimal(var1.toString()).pow(new egl.javascript.BigDecimal(var2.toString()),egl.javascript.BigDecimal.prototype.eglMC);
	}
	return Math.pow(Number(var1),Number(var2));
};

egl.convertToString = function(str) {
	//this is here only to convert a null assignment to the empty string
	return (str == null) ? "" : str.toString();
};

egl.inferSignature = function( x )
{
	if ( x == null )
	{
		return " ;";
	}
	else
	{
		switch ( typeof x )
		{
			case "string": return "?S;";
			case "number": 
				return ((""+x).indexOf('.')<0)?"?I;":"?F;";
			case "boolean": return "?0;";
			case "object":
				if ( "eze$$signature" in x && typeof x.eze$$signature == "string" )
				{
					return x.eze$$signature;
				}
				else if ( "signature" in x && typeof x.signature == "function" )
				{
					return x.signature();
				}
				else if ( x instanceof egl.javascript.BigDecimal )
				{
					var len = x.toString().length;
					var scale = x.scale();
					if ( x.signum() < 0 ) len--;
					if ( scale > 0 ) len--;
					return "?d" + len + ":" + scale + ";";
				}
				else if ( x instanceof Date )
				{
					return "?J'yyyyMMddHHmmssSSS';";
				}
				else if ( x instanceof Array )
				{
					if ( x.getType && typeof x.getType() == 'string' )
					{
						return "1" + x.getType();
					}
					
					var eltSig;
					for ( var n = 0; n < x.length; n++ )
					{
						var sig = egl.inferSignature( x[n] );
						var newEltSig = sig.slice( 0, sig.length - 1 );
						if ( !eltSig )
						{
							eltSig = newEltSig;
						}
						else if ( eltSig !== newEltSig )
						{
							eltSig = "A";
							break;
						}
					}
					if ( !eltSig )
					{
						eltSig = "A";
					}
					return "1" + eltSig + ";";
				}
				else
				{
					var classStr = Object.prototype.toString.apply( x );
					classStr = classStr.substring( 8, classStr.length - 1 );
					classStr = classStr.replace( /\./g, "/" );
					return "T" + classStr + ";";
				}
				break;
		}
		return "T<unknown type>;";
	}
};

egl.typeName = function( signature )
{
	var nullable;
	var kind;

	var firstCharIdx = 0;
	var firstChar = signature.charAt(0);
	if ( firstChar !== '?' )
	{
		nullable = '';
		kind = firstChar;
	}
	else
	{
		nullable = '?';
		kind = signature.charAt(1);
		firstCharIdx = 1;
	}

	switch ( kind )
	{
		case 'T':
			return signature.substring(1,signature.length-1).replace( /\//g, '.' );

		case '1':
			return egl.typeName(signature.substring(1,signature.length)) + '[]';

		case ' ':
			return 'null';

		case 'A':
			return 'any';

		case 'S':
			return 'string' + nullable;

		case 's':
			return 'string(' + signature.substring(firstCharIdx+1, signature.indexOf(';')) + ')' + nullable;

		case 'C':
			return 'char(' + signature.substring(firstCharIdx+1, signature.indexOf(';')) + ')' + nullable;

		case 'D':
			return 'dbchar(' + signature.substring(firstCharIdx+1, signature.indexOf(';')) + ')' + nullable;

		case 'M':
			return 'mbchar(' + signature.substring(firstCharIdx+1, signature.indexOf(';')) + ')' + nullable;

		case 'U':
			return 'unicode(' + signature.substring(firstCharIdx+1, signature.indexOf(';')) + ')' + nullable;

		case 'K':
			return 'date' + nullable;

		case 'L':
			return 'time' + nullable;

		case 'J':
			var start = signature.indexOf('\'') + 1;
			var end = signature.indexOf('\'', start);
			return 'timestamp(' + signature.substring(start, end) + ')' + nullable;

		case 'I':
			return 'int' + nullable;

		case 'i':
			return 'smallint' + nullable;

		case '0':
			return 'boolean' + nullable;

		case 'F':
			return 'float' + nullable;

		case 'f':
			return 'smallfloat' + nullable;

		case 'O':
			return 'number' + nullable;

		case 'B':
			return 'bigint' + nullable;

		case 'b':
			var colon = signature.indexOf(':');
			var decimals = signature.substring(colon+1, signature.indexOf(';'));
			var length = signature.substring(firstCharIdx+1, colon);
			if (decimals != "0")
			{
				return 'bin(' + length + ', ' + decimals + ')' + nullable;
			}
			else
			{
				if (length == "4")
				{
					return 'smallint' + nullable;
				}
				else if (length == "9")
				{
					return 'int' + nullable;
				}
				return 'bigint' + nullable;
			}

		case 'N':
			var colon = signature.indexOf(':');
			return 'num(' + signature.substring(firstCharIdx+1, colon) + ', ' + signature.substring(colon+1, signature.indexOf(';')) + ')' + nullable;

		case 'n':
			var colon = signature.indexOf(':');
			return 'numc(' + signature.substring(firstCharIdx+1, colon) + ', ' + signature.substring(colon+1, signature.indexOf(';')) + ')' + nullable;

		case 'd':
			var colon = signature.indexOf(':');
			return 'decimal(' + signature.substring(firstCharIdx+1, colon) + ', ' + signature.substring(colon+1, signature.indexOf(';')) + ')' + nullable;

		case '9':
			var colon = signature.indexOf(':');
			return 'money(' + signature.substring(firstCharIdx+1, colon) + ', ' + signature.substring(colon+1, signature.indexOf(';')) + ')' + nullable;

		case 'p':
			var colon = signature.indexOf(':');
			return 'pacf(' + signature.substring(firstCharIdx+1, colon) + ', ' + signature.substring(colon+1, signature.indexOf(';')) + ')' + nullable;

		case 'X':
			return 'hex(' + signature.substring(firstCharIdx+1, signature.indexOf(';')) + ')' + nullable;

		case 'Q':
		case 'q':
			var start = signature.indexOf('\'') + 1;
			var end = signature.indexOf('\'', start);
			return 'interval(' + signature.substring(start, end) + ')' + nullable;

		case 'l':
			return 'interval' + nullable;

		case 'W':
			return 'blob';

		case 'Y':
			return 'clob';

		case 'y':
			return 'dictionary';

		case 'a':
			return 'arraydictionary';
	}
		
	return 'unknown';
};

egl.boxAny = function( val, sig )
{
	if ( val == null || (typeof val === "object" && "eze$$value" in val && "eze$$signature" in val) )
	{
		return val;
	}

	if ( !sig )
	{
		sig = egl.inferSignature( val );
	}

	if ( sig.search( /1+A;/ ) !== -1 )
	{
		val = egl.boxElements( val, sig );
	}

	return { eze$$value : val, eze$$signature : sig };
};

egl.boxElements = function( array, sig )
{
	if ( array != null )
	{
		if ( !sig )
		{
			sig = egl.inferSignature( array );
		}

		if ( sig === "1A;" )
		{
			for ( var i = 0; i < array.length; i++ )
			{
				var boxed = egl.boxAny( array[ i ] );
				array[ i ] = boxed;
				if ( boxed != null && boxed.eze$$signature.search( /1+A;/ ) !== -1 )
				{
					egl.boxElements( boxed.eze$$value, boxed.eze$$signature );
				}
			}
		}
		else if ( sig.search( /1+A;/ ) !== -1 )
		{
			for ( var i = 0; i < array.length; i++ )
			{
				array[ i ] = egl.boxElements( array[ i ], sig.substring( 1 ) );
			}
		}
	}
	return array;
};

egl.unboxAny = function( any )
{
	if ( any == null || typeof any !== "object" || !("eze$$value" in any) || !("eze$$signature" in any) )
	{
		return any;
	}
	else
	{
		var val = any.eze$$value;
		if ( any.eze$$signature.search( /1+A;/ ) !== -1 )
		{
			val = egl.unboxElements( val, any.eze$$signature );
		}
		return val;
	}
};

egl.unboxElements = function( array, sig )
{
	if ( array == null )
	{
		return array;
	}
	
	if ( sig === "1A;" )
	{
		var unboxed = new Array( array.length );
		for ( var i = 0; i < array.length; i++ )
		{
			var boxed = array[ i ];
			if ( boxed != null )
			{
				if ( boxed.eze$$signature.search( /1+A;/ ) !== -1 )
				{
					unboxed[ i ] = egl.unboxElements( boxed.eze$$value || boxed, boxed.eze$$signature );
				}
				else
				{
					unboxed[ i ] = boxed.eze$$value || boxed;
				}
			}
			else
			{
				unboxed[ i ] = null;
			}
		}
		return unboxed;
	}
	else if ( sig.search( /1+A;/ ) !== -1 )
	{
		var unboxed = new Array( array.length );
		sig = sig.substring( 1 );
		for ( var i = 0; i < array.length; i++ )
		{
			unboxed[ i ] = egl.unboxElements( array[ i ], sig );
		}
		return unboxed;
	}
	else
	{
		return array;
	}
};

egl.isa = function( any, signature )
{
	if ( any != null )
	{
		return egl.inferSignature( any ) === signature;
	}
	else
	{
		return false;
	}
};

egl.valueString = function( any )
{
	if ( any === null )
	{
		return "null";
	}
	else if ( any.eze$$signature.charAt( 0 ) === 'T' )
	{
		return egl.typeName( any.eze$$signature );
	}
	else
	{
		return String( any.eze$$value );
	}
};

egl.convertAnyToType = function( any, sig )
{
	var isNullable = sig.charAt(0) === '?';
	var kind = sig.charAt(0) === '?' ? sig.charAt(1) : sig.charAt(0);
	switch ( kind )
	{
		case 'A':
			return any;
			
		case 'I':
			return egl.convertAnyToInt(any, isNullable);

		case 'i':
			return egl.convertAnyToSmallint(any, isNullable);

		case '0':
			return egl.convertAnyToBoolean(any, isNullable);
			
		case 'F':
			return egl.convertAnyToFloat(any, isNullable);

		case 'f':
			return egl.convertAnyToSmallfloat(any, isNullable);

		case 'B':
			return egl.convertAnyToBigint(any, isNullable);
		
		case 'N':
		case 'n':
		case 'd':
		case '9':
		case 'p':
		case 'b':
			sig = sig.charAt(0) === '?' ? sig.substring(1) : sig;
			sig = sig.substring(1);
			sig = sig.substring(0,sig.length-1);

			var parts = sig.split(/:/);
			var length=parseInt(parts[0]);
			var decimals = parseInt(parts[1]);

			var limit = "";
			for ( var len=length; len > 0; len-- ) {
			   limit += "9";
			}
			if ( decimals > 0 )	{
			   limit = limit.substring( 0, length - decimals ) + '.' + limit.substring( length - decimals );
			}
			return egl.convertAnyToDecimal(any, decimals, new egl.javascript.BigDecimal(limit), isNullable);
		
		case 'S':
		case 's':
		case 'C':
		case 'D':
		case 'M':
		case 'U':
			return egl.convertAnyToString(any, isNullable);
		
		case 'K':
			return egl.convertAnyToDate(any, isNullable);
			
		case 'L':
			return egl.convertAnyToTime(any, isNullable);
			
		case 'J':
			sig = sig.charAt(0) === '?' ? sig.substring(1) : sig;
			sig = sig.substring(1);
			sig = sig.substring(0,sig.length-1);

			var parts = sig.split(/'/);
			var pattern="";
			if (egl.IE) {
				pattern=parts[0];
			}
			else {
				pattern=parts[1];
			}
			
			return egl.convertAnyToTimestamp(any, isNullable, pattern);
			
		case 'T':
			return egl.convertAnyToNameType(any, sig);
			
		case '1':
			return egl.convertAnyToArrayType(any, sig);
	}

	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), egl.typeName( sig ) ], egl.typeName( sig ), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToRefType = function( any, sig, cons )
{
	if ( any == null )
	{
		return null;
	}
	else if ( (cons && any.eze$$value && any.eze$$value instanceof cons) || any.eze$$signature === sig )
	{
		return any.eze$$value;
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), egl.typeName( sig ) ], egl.typeName( sig ), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToArrayType = function( any, sig )
{
	if ( any == null )
	{
		return null;
	}
	else if ( any.eze$$signature === sig )
	{
		return any.eze$$value;
	}
	else
	{
		var array;
		var anyeltsig;
		if ( any instanceof Array )
		{
			array = any;
		}
		else if ( any.eze$$value instanceof Array )
		{
			array = any.eze$$value;
			if ( any.eze$$signature.charAt( 0 ) == '1' && any.eze$$signature != '1A;' )
			{
				anyeltsig = any.eze$$signature.substring(1);
			}
		}

		if ( array )
		{
			var eltsig = sig.substring(1);
			var ret = [].setType(eltsig);
			for ( var i = 0; i < array.length; i++ )
			{
				var elt = egl.boxAny( array[i], anyeltsig );
				ret[i] = egl.convertAnyToType(elt,eltsig);
			}
			return ret;
		}
	}

	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), egl.typeName( sig ) ], egl.typeName( sig ), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToNameType = function( any, sig, cons )
{
	if ( any == null )
	{
		return new cons;
	}
	else if ( any.eze$$signature === sig )
	{
		return any.eze$$value;
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), egl.typeName( sig ) ], egl.typeName( sig ), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToInt = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'I':
			case 'i':
				return any.eze$$value;

			case 'F':
			case 'f':
			case 'O':
				return egl.convertFloatToInt(any.eze$$value, egl.createTypeCastException);
			
			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return egl.convertDecimalToInt(any.eze$$value, egl.createTypeCastException);
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.convertStringToInt(any.eze$$value);
			
			case 'K':
				return egl.convertDateToInt(any.eze$$value);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'int' + (nullable ? '?' : '') ], 'int' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToSmallint = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'i':
				return any.eze$$value;

			case 'F':
			case 'f':
			case 'O':
				return egl.convertFloatToSmallint(any.eze$$value, egl.createTypeCastException);
			
			case 'I':
				return egl.convertNumberToSmallint(any.eze$$value, egl.createTypeCastException);
			
			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return egl.convertDecimalToSmallint(any.eze$$value, egl.createTypeCastException);
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.convertStringToSmallint(any.eze$$value);
			
			case 'K':
				return egl.convertNumberToSmallint(egl.convertDateToInt(any.eze$$value), egl.createTypeCastException);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'smallint' + (nullable ? '?' : '') ], 'smallint' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToBigint = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'B':
				return any.eze$$value;

			case 'I':
			case 'i':
				return new egl.javascript.BigDecimal(any.eze$$value.toString());

			case 'F':
			case 'f':
			case 'O':
				return egl.convertFloatToBigint(any.eze$$value,egl.createTypeCastException);
			
			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return egl.convertDecimalToBigint(any.eze$$value,egl.createTypeCastException);
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.convertStringToBigint(any.eze$$value);
			
			case 'K':
				return new egl.javascript.BigDecimal(egl.convertDateToInt(any.eze$$value).toString());
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'bigint' + (nullable ? '?' : '') ], 'int' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToString = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : '';
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return any.eze$$value;
			
			case 'K':
				return egl.egl.core.$StrLib.formatDate(any.eze$$value,egl.egl.core.$StrLib.defaultDateFormat);
			
			case 'L':
				return egl.egl.core.$StrLib.formatTime(any.eze$$value,egl.egl.core.$StrLib.defaultTimeFormat);
			
			case 'J':
				var pattern;
				if ( egl.egl.core.$StrLib.defaultTimeStampFormat.length === 0 )
				{
					pattern = any.eze$$signature.substring( any.eze$$signature.charAt(0) !== '?' ? 2 : 3, any.eze$$signature.length - 2 );
				}
				else
				{
					pattern = egl.egl.core.$StrLib.defaultTimeStampFormat;
				}
				return egl.egl.core.$StrLib.formatTimeStamp(any.eze$$value,pattern);

			case 'I':
			case 'i':
			case '0':
			case 'F':
			case 'f':
			case 'O':
			case 'B':
			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
				return any.eze$$value.toString();
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'string' + (nullable ? '?' : '') ], 'string' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToDecimal = function( any, decimals, limit, nullable )
{
	if ( any == null )
	{
		return nullable ? null : egl.javascript.BigDecimal.prototype.ZERO;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'I':
			case 'i':
				return egl.convertIntegerToDecimal(any.eze$$value,limit,egl.createTypeCastException);

			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return egl.convertDecimalToDecimal(any.eze$$value,decimals,limit,egl.createTypeCastException);

			case 'F':
			case 'f':
			case 'O':
				return egl.convertFloatToDecimal(any.eze$$value,decimals,limit,egl.createTypeCastException);

			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.convertStringToDecimal(any.eze$$value,decimals,limit);
			
			case 'K':
				return egl.convertIntegerToDecimal(egl.convertDateToInt(any.eze$$value),limit,egl.createTypeCastException);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'decimal' + (nullable ? '?' : '') ], 'decimal' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToFloat = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'I':
			case 'i':
			case 'F':
			case 'f':
			case 'O':
				return any.eze$$value;
			
			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return Number( any.eze$$value.toString() );
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.convertStringToFloat(any.eze$$value);
			
			case 'K':
				return egl.convertDateToInt(any.eze$$value);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'float' + (nullable ? '?' : '') ], 'float' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToSmallfloat = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'I':
			case 'i':
			case 'f':
				return any.eze$$value;
			
			case 'F':
			case 'O':
				return egl.convertFloatToSmallfloat(any.eze$$value, egl.createTypeCastException);
			
			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return egl.convertFloatToSmallfloat(Number(any.eze$$value.toString()), egl.createTypeCastException);
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.convertFloatToSmallfloat(egl.convertStringToFloat(any.eze$$value), egl.createTypeCastException);
			
			case 'K':
				return egl.convertDateToInt(any.eze$$value);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'smallfloat' + (nullable ? '?' : '') ], 'smallfloat' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToBoolean = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : false;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case '0':
				return any.eze$$value;

			case 'I':
			case 'i':
			case 'F':
			case 'f':
			case 'O':
				return Boolean(any.eze$$value);

			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return Boolean(!any.eze$$value.equals(egl.javascript.BigDecimal.prototype.ZERO));
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'boolean' + (nullable ? '?' : '') ], 'boolean' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToDate = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'K':
			case 'J':
				return any.eze$$value;

			case 'I':
			case 'i':
				return egl.egl.core.$DateTimeLib.dateFromInt(any.eze$$value);

			case 'F':
			case 'f':
			case 'O':
				return egl.egl.core.$DateTimeLib.dateFromInt(egl.convertFloatToInt(any.eze$$value));

			case 'N':
			case 'n':
			case 'd':
			case '9':
			case 'p':
			case 'B':
				return egl.egl.core.$DateTimeLib.dateFromInt(egl.convertDecimalToInt(any.eze$$value));
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.egl.core.$DateTimeLib.dateValue(any.eze$$value);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'date' + (nullable ? '?' : '') ], 'date' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToTime = function( any, nullable )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'L':
			case 'J':
				return any.eze$$value;
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.egl.core.$DateTimeLib.timeValue(any.eze$$value);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'time' + (nullable ? '?' : '') ], 'time' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

egl.convertAnyToTimestamp = function( any, nullable, pattern )
{
	if ( any == null )
	{
		return nullable ? null : 0;
	}
	else
	{
		var kind = any.eze$$signature.charAt(0) === '?' ? any.eze$$signature.charAt(1) : any.eze$$signature.charAt(0);
		switch ( kind )
		{
			case 'J':
				return egl.egl.core.$DateTimeLib.extend("timeStamp",any.eze$$value,pattern);

			case 'K':
				return egl.egl.core.$DateTimeLib.extend("date",any.eze$$value,pattern);

			case 'L':
				return egl.egl.core.$DateTimeLib.extend("time",any.eze$$value,pattern);
			
			case 'S':
			case 's':
			case 'C':
			case 'D':
			case 'M':
			case 'U':
				return egl.egl.core.$DateTimeLib.timeStampValueWithPattern(any.eze$$value,pattern);
		}
	}
	throw egl.createTypeCastException( "CRRUI2017E", [ egl.valueString( any ), egl.typeName( any.eze$$signature ), 'timestamp' + (nullable ? '?' : '') ], 'timestamp' + (nullable ? '?' : ''), egl.typeName( any.eze$$signature ) );
};

Array.prototype.maxSize=2147483647;

Array.prototype.appendElement = function Array_appendElement(val) {
	if ( this.length < this.maxSize )
	{
		this.push( val );
		return this;
	}
	else
	{
		throw egl.createIndexOutOfBoundsException( 'CRRUI2019E', [ val, this.maxSize ] );
	}
};	

Array.prototype.setType = function (type) { 
	this.type = type; 
	return this;
};

Array.prototype.getType = function () { return this.type; };

Array.prototype.assign = function Array_assign(other) {
	this.length = 0;
	for (var n=0; n<other.length; n++) this.push(other[n]);
}

Array.prototype.appendAll = function Array_appendAll(other) {
	for (var n=0; n<other.length; n++) this.push(other[n]);
	return this;
};

Array.prototype.getSize = function Array_getSize() {
	return this.length;
};

Array.prototype.contains = function Array_contains() {
	if (arguments.length > 2)
	{
		return this[arguments[1]]["contains"](arguments[0],arguments[2]);
	}
	var searchObject = arguments[0];
	var startingIndex = 0;
	if (arguments.length==2)
		startingIndex=arguments[1]-1;
	if (startingIndex < 0)
		startingIndex=0;
	for (var n=startingIndex; n < this.length; ++n) {
		if ( egl.unboxAny(this[n]) == searchObject ) {
			return true;
		}
	}
	return false;
};

Array.prototype.insertElement = function Array_insertElement(val, index) {
	//Remember EGL is 1-based, JS 0-based
	if ( index < 1 || index > this.length + 1 ) 
		throw egl.createRuntimeException( "CRRUI2022E", [ index, this.length ] );
	if ( this.length < index ) {
		this.push( val );
		return this;
	}
	this.splice( index-1, 0, val );
	return this;
}

Array.prototype.removeAll = function Array_removeAll(val) {
	this.length = 0;
};

Array.prototype.removeElement = function Array_removeElement(index) {
	this.splice( index-1, 1 );
};


egl.initialValueForType = function ( elementType ) 
{
	var kind = elementType.charAt(0);
	switch ( kind )
	{
		case 'A':
		case '?':
			return null;
		case 'I':
		case 'i':
		case 'F':
		case 'f':
		case 'O':
			return 0;
		case 'S':
		case 'U':
		case 'C':
		case 's':
			return "";
		case '0':
			return false;
		case 'd':
		case 'N':
		case '9':
		case 'B':
			return egl.javascript.BigDecimal.prototype.ZERO;
		case 'K':
			return egl.egl.core.$DateTimeLib.currentDate();
		case 'L':
		case 'J':
			return egl.egl.core.$DateTimeLib.currentTime();
		default:
			throw egl.createRuntimeException( "CRRUI2034E", [ elementType ] );
	}
};

Array.prototype.resize = function Array_resize(val) {
	if ( val > this.maxSize )
	{
		throw egl.createRuntimeException( "CRRUI2033E", [ val, this.maxSize ] );
	}
	if (val <= this.length) {
		this.length = val;
		return;
	}
	var elementType = this.getType();
	
	for (var n=this.length; n < val; n++) {
		if ( typeof elementType == 'function')
		{
			var obj = new elementType();
			if ( !(obj instanceof egl.egl.jsrt.Record) )
				this.push( null );
			else
				this.push( obj );
		}
		else if ( typeof elementType == 'string' )
		{
			this.push( egl.initialValueForType( elementType ) );
		}
		else if ( elementType == null )
		{
			this.push( null );
		}
		else
		{
			this.push( {} );
		}
	}
};

egl.resizeAll = function egl_resizeAll(array, sizes) {
	if (array) {  
		if ( sizes.length == 1 )
		{
			array.resize( sizes[0] );
		}
		else
		{
				// resize this level of the array
			if ( array.length >= sizes[0] )
			{
				array.length = sizes[0];
			}
			else
			{
				var arrayToAdd = sizes[0] - array.length;
				for ( var i = 0; ( i < arrayToAdd ); i++ )
				{
					var newArray = [];
					newArray.setType( array.getType() );
					newArray.maxSize = array[0].maxSize;
					array.push( newArray );
				} 
			}
				// delegate to the next level of resizeAll
			for ( var i = 0; ( i < array.length ); i++ )
			{
				egl.resizeAll( array[i], sizes.slice(1) );
			}
		}
	}
	else
	{
		throw egl.createRuntimeException( "CRRUI2035E", null );
	}
}

egl.move = function egl_move(source, target, byName, byPosition, forAll, count) {
	for (f in source) {
		if (!egl.isUserField(source, f))
			continue;
		if (typeof(source[f])=="object") {
			egl.move(source[f], target[f] = {});
		}
		else {
			target[f] = source[f];
		}
	}
	return source;
}

Array.prototype.resizeAll = function Array_resizeAll(sizes) {
	egl.resizeAll(this, sizes);
};

Array.prototype.setMaxSize = function Array_setMaxSize(val) 
{
	if ( val < 0 || this.length > val )
	{
		throw egl.createRuntimeException( 'CRRUI2020E', [ val ] );
	}
	this.maxSize = val;
};

Array.prototype.getMaxSize = function Array_getMaxSize() {
	return this.maxSize;
};

Array.prototype.setMaxSizes = function Array_setMaxSizes(sizes) {
	this.setMaxSize( sizes[0]);
	if ( sizes.length > 1 ) {
		for (var n=0; n<this.length; n++) {
			this[n].setMaxSizes( sizes.slice(1) );
		}
	}
};

Array.prototype.eze$$getName = function() {
	return "";
}

Array.prototype.eze$$getChildVariables = function Array_getChildVaribles_for_debug_support() {
	var childVars = [];
	for (var n=0; n<this.length; n++) {
		childVars[n] = {name : "[" + n + "]", value : this[n]};
	}
	return childVars;
};

egl.defineClass(
	'egl.core', 'SysLib',
{
	"constructor" : function() {
		egl.egl.core.SysLib.$inst = this;
		var core=egl.egl.core;
		if (egl.egl.core.$SysLib) return egl.egl.core.$SysLib;
		egl.egl.core.$SysLib=this;
		
		this.errorsMap = [];
	},
	"set" : function( /*Object*/ ref, /*value*/ val ) {
		if ( ref instanceof String || ref instanceof Boolean || ref instanceof Number || ref instanceof Date) {
			throw egl.createRuntimeException( "CRRUI2099E", [ typeof(ref) ] );
		}
		else
		if ( ref instanceof Array ) {
			for (var index = 0; index < ref.length; index++ ) {
				ref[index] = val;
			}
		}
		else {
			for (var field in ref) {
				if (!field in Object.prototype) {
					ref[index] = val;
				}
			}
		}
	},	
	"conditionAsInt" : function(/*boolean*/val) {
		//Returns the integer 1 or the integer 0 based on a Boolean value
		return val ? 1 : 0;
	},
	"getProperty" : function(property, propertyFile) {
		var value;
		if (egl.eze$$runtimeProperties[propertyFile]) {
			value = egl.eze$$runtimeProperties[propertyFile][property];
		}
		
		if (!value && egl.eze$$runtimeProperties["rununit"]) {
			value = egl.eze$$runtimeProperties["rununit"][property];
		}
		
		if (value === undefined) {
			value = null;
		}
		return value;
	},
	"eze$$getName" : function() {
		return "egl.core.SysLib";
	},
	"eze$$getChildVariables": function() {
		return [];
	}
});

//Wrapper for internal validators that need extra parameters.
egl.defineClass(
		'egl.rui.mvc.internal', 'ValidatorWrapper',
{
	"constructor" : function( context, func ) {
		this.context = context;
		this.func = func;
		this.params = [];
		for (var i = 2; i < arguments.length; i++) {
			this.params.push(arguments[i]);
		}
	},
	"validate" : function( value ) {
		var newParams = this.params.slice();
		newParams.splice(0, 0, value);
		return this.func.apply(this.context, newParams);
	}
});

egl.defineClass(
		'egl.rui.mvc.internal', 'FormatterWrapper',
{
	"constructor" : function( context ) {
		this.context = context;
		this.params = [];
		for (var i = 1; i < arguments.length; i++) {
			this.params.push(arguments[i]);
		}
	},
	"format" : function( value ) {
		var newParams = this.params.slice();
		newParams.splice(0, 0, value);
		return this.context.format.apply(this.context, newParams);
	},
	"unformat" : function( value ) {
		var newParams = this.params.slice();
		newParams.splice(0, 0, value);
		return this.context.unformat.apply(this.context, newParams);
	}
});

egl.defineClass( "egl.core", "StrLib", {
	"constructor" : function() {
		if (egl.egl.core.$StrLib) return egl.egl.core.$StrLib;
		
		egl.egl.core.StrLib.$inst = this;
		egl.egl.core.$StrLib=this;
		
		// system "constants"
		this.db2TimestampFormat = "yyyy-MM-dd-HH.mm.ss.SSSSSS";
		this.eurDateFormat = "dd.MM.yyyy";
		this.eurTimeFormat = "HH.mm.ss";
		this.isoDateFormat = "yyyy-MM-dd";
		this.isoTimeFormat = "HH.mm.ss";
		this.jisDateFormat = "yyyy-MM-dd";
		this.jisTimeFormat = "HH:mm:ss";
		this.odbcTimestampFormat = "yyyy-MM-dd HH:mm:ss.SSSSSS";
		this.usaDateFormat = "MM/dd/yyyy";
		this.usaTimeFormat = "hh:mm a";

		// system variables
		this.defaultDateFormat = "MM/dd/yyyy";
		this.defaultMoneyFormat = "";
		this.defaultNumericFormat = "";
		this.defaultTimeFormat = "HH:mm:ss";
		this.defaultTimeStampFormat = "";
	},
	
	"booleanAsString" : function(/*boolean*/val) {
		//Returns the string ?true? or the string ?false? based on a Boolean value
		return val.toString();
	},
	
	"byteLen" : function(/*string*/ text) {
		//applicable only for strings, unicodes, and string literals
		return 2 * this.textLen(text)
	},
	
	"charAsInt" : function(/*string*/val) {
		return (val.charCodeAt());
	},
	
	"clipBack" : function(/*string*/str) {
		return str.replace( /(\s)*$/, "" );
	},
	
	"clipFront" : function(/*string*/str) {
		return str.replace( /^(\s)*/, "" );
	},
	
	"clip": function(/*string*/str, /*int*/ option) {
		//checks against a null any type
		if ( str == null )
			return null;
		
		//option = 0 - leading and trailing
		//option = 1 - leading only
		//option = 2 - trailing only		
		var clipped = str;
		switch (option) {
		case 0:
			clipped = this.clipFront(clipped);
			clipped = this.clipBack(clipped);
			break;
		case 1:
			clipped = this.clipFront(clipped);
			break;
		default:
			clipped = this.clipBack(clipped);
			break;
		}
		return clipped;
	},
	
	/*
	 * Formats a number, returning a string.
	 */
	"formatNumber" : function(/*number*/n, /*string*/format) {
//FIXME implement formatNumber
	},
	
	/*
	 * Formats a date value, returning a STRING.
	 */
	"formatDate" : function(/*date*/date, /*optional string*/format) {
		if (!format) {
			format = egl.egl.core.$StrLib.defaultDateFormat;
		}
		
		try
		{
			return egl.dateToString( date, format );
		}
		catch ( oops )
		{
			return null;
		}
	},
	
	/*
	 * Formats a time value, returning a STRING.
	 */
	"formatTime" : function(/*time*/val, /*optional string*/format) {
		if (!format) {
			format = egl.egl.core.$StrLib.defaultTimeFormat;
		}
		try
		{
			return egl.timeToString( val, format );
		}
		catch ( oops )
		{
			return null;
		}
	},
	
	/*
	 * Formats timestamp value, returning a STRING.
	 * The DB2 format is the default format.
	 */
	"formatTimeStamp" : function(/*timestamp*/val, /*optional string*/format) {
		if (!format || format == "" ) {
			format = this.db2TimestampFormat;
		}
		try
		{
			return egl.timeStampToString( val, format );
		}
		catch ( oops )
		{
			return null;
		}
	},
	
	"getNextToken" : function(/*string*/source, /*int*/index, /*string*/delimiters, outAssignFn) {
		// Adjust for EGL's 1-based indexing
		var startIndex = (index - 1) / 2;
		
		//Skip leading tokens
		while(delimiters.indexOf(source.charAt(startIndex)) >= 0 && startIndex < source.length)
			startIndex++;
		
		// If we're at the end of the substring, the search failed
		if(startIndex >= source.length) {
			throw egl.createIndexOutOfBoundsException("CRRUI2021E", [ index ], index );
		}
			
		// Now we know we've found the beginning of a token. Find its end.
		var tokenEnd = startIndex + 1;

		// Check each byte to see if it's the start of a one-byte delimiter.
		while(delimiters.indexOf(source.charAt(tokenEnd)) < 0 && tokenEnd < source.length)
			tokenEnd++;

		// Store the end of the token in index and adjust for doubling.
		index = (tokenEnd * 2) + 1;

		if(outAssignFn)
			outAssignFn(source,index);
				
		return source.slice(startIndex,tokenEnd);
	},
	
	"getTokenCount" : function(/*string*/ source, /*string*/ delimiters) {
		//Returns the number of tokens in a source string by simulating the getTokenCount function.
		var index = 1;
		var tokens;
		var next;
		var endAssign = function(updatedsource,updatedindex) { source = updatedsource; index = updatedindex; };
		
		try {
			for ( tokens = 0; tokens < source.length; tokens++ ) {
				next = this.getNextToken( source, index, delimiters, endAssign );
			}
		}
		catch ( exception ) {
			//out of tokens
		}
		finally {
			return tokens;
		}
	},
	
	"indexOf" : function( /*string*/s, /*string*/pattern, /*optional int*/startPosition) {
		//Returns an index into a string of characters indicating where a specified pattern begins.
		try {
			if (startPosition) startPosition = startPosition-1;
			else startPosition = 0;
			return s.indexOf(pattern, startPosition) + 1;
		}
		catch (e) {
			throw egl.createRuntimeException( "CRRUI2101E", [ s, pattern, e.message] );
		} 
	},
	
	"intAsChar" : function(/*integer*/i) {
		//Returns a character corresponding to an integer code point.
		return (i == null) ? null : String.fromCharCode(i);
	},
	
	"integerAsString" : function(/*integer*/i) {
		//Returns a string corresponding to an integer.
		return (String.fromCharCode(i));
	},
	
	"intAsUnicode" : function(/*integer*/i) {
		//Returns a Unicode character corresponding to an integer code point.
		return (i == null) ? null : String.fromCharCode(i);
	},
	
	"lowerCase" : function(/*string*/s) {
		//Converts all uppercase values in a character string to lowercase values.
		return s.toLowerCase();
	},
	
	"setBlankTerminator" : function( /*string*/s )  {
		//Replaces a null terminator and any subsequent characters in a string with spaces.
	},
	
	"setNullTerminator" : function(/*string*/s) {
		//Changes all trailing spaces in a string to null values.
	},
	
	"spaces" : function(/*integer*/i) {
		//Returns a string of a specified length.
		var s = [];
		for (var n=0;n<i;++n) {
			s.push(' ');
		}
		return (s.join(''));
	},
	
	"textLen" : function(/*string*/s) {
		//Returns the number of bytes in a text expression, excluding any trailing spaces or null values.
		if ( s === null )
			return 0;
		return this.clip(s).length;
	},
	
	"unicodeAsInt" : function(/*integer*/i) {
		//Returns an integer corresponding to a Unicode character.
		return i.charCodeAt();
	},
	
	"upperCase" : function(/*string*/s) {
		//Converts all lowercase values in a character string to uppercase values. Numeric and existing uppercase values are not affected.
		return s.toUpperCase();
	},
	
	"eze$$getName" : function() {
		return "egl.core.StrLib";
	},
	
	"eze$$getChildVariables": function() {
		return [
		{name: "db2TimestampFormat", value : this.db2TimestampFormat, type : "char(26)"},
		{name: "defaultDateFormat", value : this.defaultDateFormat, type : "string"},
		{name: "defaultMoneyFormat", value : this.defaultMoneyFormat, type : "string"},
		{name: "defaultNumericFormat", value : this.defaultNumericFormat, type : "string"},
		{name: "defaultTimeFormat", value : this.defaultTimeFormat, type : "string"},
		{name: "defaultTimeStampFormat", value : this.defaultTimeStampFormat, type : "string"},
		{name: "eurDateFormat", value : this.eurDateFormat, type : "char(10)"},
		{name: "eurTimeFormat", value : this.eurTimeFormat, type : "char(8)"},
		{name: "isoDateFormat", value : this.isoDateFormat, type : "char(10)"},
		{name: "isoTimeFormat", value : this.isoTimeFormat, type : "char(8)"},
		{name: "jisDateFormat", value : this.jisDateFormat, type : "char(10)"},
		{name: "jisTimeFormat", value : this.jisTimeFormat, type : "char(8)"},
		{name: "odbcTimestampFormat", value : this.odbcTimestampFormat, type : "char(26)"},
		{name: "usaDateFormat", value : this.usaDateFormat, type : "char(10)"},
		{name: "usaTimeFormat", value : this.usaTimeFormat, type : "char(7)"}
		];
	}
});

egl.defineClass( "egl.core", "DateTimeLib", {
	"constructor" : function() {
		egl.egl.core.DateTimeLib.$inst = this;
		var core=egl.egl.core;
		if (egl.egl.core.$DateTimeLib) return egl.egl.core.$DateTimeLib;
		egl.egl.core.$DateTimeLib=this;
	},
	
	"currentDate" : function() {		
		//returns a new Date object
		//just ignore time fields
		return new Date();
	},
	
	"currentTime" : function() {
		//returns a new Date object
		//just ignore date fields
		return new Date();
	},
	
	"currentTimeStamp" : function( /*string*/ pattern ) {
		//returns a new Date object
		//timestamp keeps every field
		return this.extend( "timestamp", new Date(), pattern );
	},
	
	"dateFromInt" : function (/*Integer*/ day ) {
		//day is an integer, where 0 represents 12/31/1899
		//a null day means return the current date
		var date = null;
		if ( day != null) {
			date = this.currentDate();
			date.setHours( 0 );
			date.setMinutes( 0 );
			date.setSeconds( 0 );
			date.setMilliseconds( 0 );
			date.setYear( 1899 );
			date.setMonth( 11 );
			date.setDate( 31 + day );		//the critical piece; days adjust schedule
		}
		return date;
	},

	"dateOf" : function(/*TimeStamp*/ ts) {
		//returns a copy of the date
		return ( (ts) ? new Date( ts ) : null );
	},

	"dateValue" : function(/*String*/ val) {
		//Returns a DATE value that corresponds to an input string.
		var d = null;
		
		if (val!=null) {
			d = egl.stringToDate( val, egl.egl.core.$StrLib.defaultDateFormat );
		}
		return d;
	},

	"dateValueFromGregorian" : function(/*Integer*/ val) {
		//returns the date based on an int in gregorian form
		if (val!=null) {
			//yyyyMMdd is the format of the int, so mod away parts
			var day = val % 100;
			val = Math.floor( val / 100 );
			var month = (val % 100) - 1;
			var year = Math.floor( val / 100 );
			return new Date( year, month, day );
		}
		else {
			return null;
		}
	},

	"dateValueFromJulian" : function(/*Integer*/ julianIntDate) {
		//Returns a DATE value that corresponds to an integer representation of a Julian date.
		//julianIntDate is of the form yyyyDDD (DDD = number of total days so far in the year)
		if (julianIntDate!=null) {
			//taking old code for what it's worth
			var year = Math.floor(julianIntDate / 1000)
			var cal = new Date( year, 0, 1 );
			cal.setTime( cal.getTime() + ( (julianIntDate % 1000) - 1 ) * egl.MS_PER_DAY );
			return cal;
		}
		else {
			return null;
		}
	},
	
	"dayOf" : function(/*TimeStamp*/ ts) {
		//returns the day of the month from the Date object
		return ( (ts) ? ts.getDate() : null );
	},
	
	"extend" : function(/*type of date*/ type, /*extension*/ date, /*optional mask*/pattern ) {
		//Converts a timestamp, time, or date into a longer or shorter timestamp value.
		if ( !date ) {
			return null;
		}
		
		//copy is returned if the pattern is invalid
		var dateCopy = new Date( date );
		var now = new Date();
		type = type.toLowerCase();
		
		if ( type == "date" ) {
			date.setHours( 0 );
			date.setMinutes( 0 );
			date.setSeconds( 0 );
			date.setMilliseconds( 0 );
			if (!pattern || pattern == "" )
				pattern = "yyyyMMddHHmmss";
		}
		else if ( type == "time" ) {
			date.setFullYear( now.getFullYear() );
			date.setMonth( now.getMonth() );
			date.setDate( now.getDate() );
			if (!pattern || pattern == "" )
				pattern = "yyyyMMddHHmmss";
		}
		else { // ( type == "timestamp" )
			if (!pattern || pattern == "" )
				pattern = "yyyyMMddHHmmssffffff";
		}
		
		//enforce the pattern mask
		//first function is for pattern missing chars on the left side (current)
		//second function is for pattern missing chars on the right side (zeros)
		var chars = 
			[
			  [ "y", function(d){ d.setFullYear( now.getFullYear() ); }, function(d){ d.setFullYear( 0 ); } ],
		      [ "M", function(d){ d.setMonth( now.getMonth() ); }, function(d){ d.setMonth( 0 ); } ], 
		      [ "d", function(d){ d.setDate( now.getDate() ); }, function(d){ d.setDate( 1 ); } ],
		      [ "H", function(d){ d.setHours( now.getHours() ); }, function(d){ d.setHours( 0 ); } ],
		      [ "m", function(d){ d.setMinutes( now.getMinutes() ); }, function(d){ d.setMinutes( 0 ); } ],
		      [ "s", function(d){ d.setSeconds( now.getSeconds() ); }, function(d){ d.setSeconds( 0 ); } ], 
		      [ "f", function(d){ d.setMilliseconds( now.getMilliseconds() ); }, function(d){ d.setMilliseconds( 0 ); } ]
		    ];
		
		//set the new date values to the current time until the first
		//char in the formatting string appears
		var leadChar = pattern.charAt( 0 );
		var i = 0;
		while ( i < chars.length && leadChar != chars[ i ][ 0 ] ) {
			(chars[ i ][ 1 ])( dateCopy );
			i++;
		}
		//if the pattern has bad characters, just return the original date
		if ( i >= chars.length ) {
			return date;
		}
		
		//find the last character and set everything after it to zeros
		var lastChar = pattern.charAt( pattern.length - 1 );
		i = chars.length - 1;
		while ( i >= 0 && lastChar != chars[ i ][ 0 ] ) {
			(chars[ i ][ 2 ])( dateCopy );
			i--;
		}
		//if the pattern has bad characters, just return the date
		if ( i < 0 ) {
			return date;
		}
		
		return dateCopy;
	},
		
	"_digitLength" : function(/*string*/pattern, /*char*/specifier, /*int*/ index ) {
		var count = 0;
		var start = pattern.indexOf( specifier, index );
		if ( start > -1 ) {
			for ( var i = start; i < pattern.length; i++ ) {
				if ( pattern.charAt( i ) == specifier ) {
					count++;
				} else {
					break;
				}
			}
		}
		return count;
	},
	
	/**
	 * Calculates the maximum number of months that can be stored in
	 * the given month-span interval.
	 * 
	 * @param pattern  Interval pattern
	 * @return the maximum number of months
	 */
	"_maxMonthValue" : function(/*String*/ pattern ) {
		var maxValue = 0;
		pattern = pattern.toLowerCase();
		var firstY = pattern.indexOf( 'y' );
		var firstM = pattern.indexOf( 'm' );
		if ( firstM >= 0 ) {
			// Pattern contains y's and M's
			if ( firstY >= 0 ) {
				maxValue = 11;
			} else {
				// Pattern only contains M's
				var lastM = pattern.lastIndexOf( 'm' );
				maxValue = this._maxValue( lastM - firstM + 1 );
			}
		}
		
		if ( firstY >= 0 )
		{	
			var lastY = pattern.lastIndexOf( 'y' );
			maxValue += this._maxValue( lastY - firstY + 1 ) * 12;
		}
		
		return maxValue;
	},
	
	/**
	 * Calculates the maximum number of seconds that can be stored in
	 * the given second-span interval. This tries to eliminate as much math
	 * as possible.
	 * 
	 * @param pattern  Interval pattern
	 * @return the maximum number of seconds
	 */
	"_maxSecondValue" : function( /*String*/ pattern ) {
		if ( pattern == null || pattern.length == 0 )
		{
			return 0;
		}
		
		var multiplier;
		pattern = pattern.toLowerCase();
		var firstChar = pattern.charAt( 0 );
		switch ( firstChar )
		{
			case 'd': multiplier = /*SECONDS_PER_DAY*/86400;
				break;
			case 'h': multiplier = 3600;
				break;
			case 'm': multiplier = 60;
				break;
			case 's': multiplier = 1;
				break;
			default: multiplier = 0;
				break;
		}
		
		return (this._maxValue( pattern.lastIndexOf( firstChar ) + 1 ) + 1)
				* multiplier - 1;
	},
	
	"_maxValue" : function(/*int*/ digits ) {
		switch( digits )
		{
			case 1: return 9;
			case 2: return 99;
			case 3: return 999;
			case 4: return 9999;
			case 5: return 99999;
			case 6: return 999999;
			case 7: return 9999999;
			case 8: return 99999999;
			case 9: return 999999999;
			default: return 0;
		}
	},
	
	"mdy" : function(/*integer*/month, /*integer*/day, /*integer*/year) {
		if(year && month && day)
		{
			var result = new Date(1970,0,1);
			result.setFullYear(year);
			result.setMonth(month-1);
			result.setDate(day);
			result.setHours(0);
			result.setMinutes(0);
			result.setSeconds(0);
			result.setMilliseconds(0);
			return result;
		}
		else
		{
			return null;
		}
	},
	
	"monthOf": function(/*timestamp*/ ts) {
		return ( (ts) ? (ts.getMonth() + 1) : null );
	},
	
	"timeOf" : function(/*optional timestamp*/ ts) {
		//returns a copy of the Date object
		return ( (ts) ? new Date( ts ) : null );
	},
	
	"timestampFrom": function(/*date*/date, /*time*/time) {
		if ( !date || !time ) {
			return null;
		}
		
		//first create a new Date object that is a copy of the date argument
		var stamp = new Date( date );
		//next, copy all of the time values over to that date copy
		var timeCal = new Date(stamp.getMilliseconds());
		timeCal.setTime( time );
		stamp.setHours( timeCal.getHours() );
		stamp.setMinutes( timeCal.getMinutes() );
		stamp.setSeconds( timeCal.getSeconds() );
		stamp.setMilliseconds( 0 );
		
		return stamp;
	},
	
	"timeStampValue" : function(/*string*/val) {
		//Returns a TIMESTAMP value that reflects a string constant or literal.
		//calling this function sets the pattern to yyyyMMddHHmmss by default
		return this.timeStampValueWithPattern( val, "yyyyMMddHHmmss" );
	},
	
	"timeStampValueWithPattern" : function(/*string*/source, /*optional mask*/timeSpanPattern) {
		//Returns a TIMESTAMP value that reflects a string and is built based on a timestamp mask that you specify
		timeSpanPattern = egl.getProperTimeStampPattern( timeSpanPattern );
		return ( ( source != null ) ? egl.stringToTimeStamp( source, timeSpanPattern ) : null );
	},

	"timeValue": function(/*string*/val) {
		//Returns a TIME value that reflects a string constant or literal.
		return ( (val != null) ? egl.stringToTime( val, egl.egl.core.$StrLib.defaultTimeFormat ) : null );
	},
	
	"weekdayOf": function(/*timestamp*/ts) {
		return ( (ts) ? ts.getDay() : null );
	},
	
	"yearOf": function(/*timestamp*/ts) {
		return ( (ts) ? ts.getFullYear() : null );
	}
	
});

egl.getProperTimeStampPattern = function( pattern ) {
	return ( pattern ) 
				? egl.formatTimeStampPattern( pattern )
				: (( egl.egl.core.$StrLib.defaultTimeStampFormat == "" ) 
						? egl.egl.core.$StrLib.db2TimestampFormat
						: egl.egl.core.$StrLib.defaultTimeStampFormat );
};

egl.formatTimeStampPattern = function( mask ) {
	// Masks specify fractions of seconds with "f", but formatting patterns use "S".
	mask = mask.replace(/f/g, "S");

	// Masks can be any subset of "yyyyMMddHHmmssffffff" (though they can't start or stop in the middle of a character block).
	// We want to return the corresponding subset of the format "yyyy-MM-dd HH:mm:ss.SSSSSS" (the odbc timestamp format)
	// that matches the subset of characters in the mask (i.e. if the mask is "MMddHH", we want to return the corresponding
	// substring of the pattern: "MM-dd HH".
	var firstCharInMask = mask.charAt(0);
	var lastCharInMask = mask.charAt(mask.length-1);
	
	var startIndex = egl.egl.core.$StrLib.odbcTimestampFormat.indexOf(firstCharInMask);
	var endIndex = egl.egl.core.$StrLib.odbcTimestampFormat.lastIndexOf(lastCharInMask)+1; // (add 1 so we actually *get* the last character)
	
	if( startIndex >= 0 && endIndex > startIndex )
	{
		return egl.egl.core.$StrLib.odbcTimestampFormat.substring( startIndex, endIndex );
	}
	else
	{
		throw egl.createRuntimeException( "CRRUI2032E", [ mask ] );
	}
};

egl.defineClass( "egl.core", "SysVar", {
	"constructor" : function() {
		this.arrayIndex = (0);
		this.overflowIndicator = (0);
		this.systemType = egl.setSystemTypeToDebug ? "DEBUG" : "JAVASCRIPT";		//cannot easily make this a constant; rely on validation
	},
	
	"eze$$getName" : function() {
		return "egl.core.SysVar";
	},
	"eze$$getChildVariables": function() {
		return [
		{name: "arrayIndex", value : this.arrayIndex, type : "int"},
		{name: "overflowIndicator", value : this.overflowIndicator, type : "num(1, 0)"},
		{name: "systemType", value : this.systemType, type : "char(8)"}
		];
	}
	
});

egl.eq = function(a,b) { return a==b; };
egl.le = function(a,b) { return a<=b; };
egl.lt = function(a,b) { return a<b; };
egl.ge = function(a,b) { return a>=b; };
egl.gt = function(a,b) { return a>b; };
egl.ne = function(a,b) { return a!=b; };
egl.neg = function(v) {	return -v; };
egl.nullableNeg = function(v) {	
	if (v==null)
		return null;
	if (v instanceof egl.javascript.BigDecimal)
		return v.negate();
	return -v; 
};
egl.not = function(b) {	return !b; };
egl.like = function(l,r,e) { if (l==null || r==null) {return false;} return l.like(r,e); };
egl.matches = function(l,r,e) { if (l==null || r==null) {return false;} return l.matches(r,e); };
egl.nullableSubstring = function(s, i1, i2) { return (s==null || i1==null || i2==null) ? null:s.substring(i1-1,i2); };
egl.nullableCheckIndex = function(s, i1) { return (s==null || i1==null) ? null:s[s.checkIndex(i1-1)]; };
egl.nullableGetRow = function(s, i1) { return (s==null || i1==null) ? null:s.getRow(i1-1); };

egl.nullableCompare = function(v1, v2, falseAnswer) {	
	if (v1==null || v2==null)
	{
		return falseAnswer; 
	}

	if (v1 instanceof egl.javascript.BigDecimal)
		if (v2 instanceof egl.javascript.BigDecimal)
			return v1.compareTo(v2);
		else
			return v1.compareTo(new egl.javascript.BigDecimal(v2.toString()));
	else if (v2 instanceof egl.javascript.BigDecimal)
		return new egl.javascript.BigDecimal(v1.toString()).compareTo(v2);

	if (v1 == v2)
		return 0;
	return (v1 > v2) ? 1 : -1; 
};

//this is a simple version for "systemType is systemName"
//the generator spits out a string literal for the systemName (since sysVar.systemType is a string)
//e is a potential comparator.  if none is provided, == operator used
egl.is = function(l,r,e) { return (e) ? e(l,r) : (l == r); };

egl.dateEquals = function(/*date*/ a, /*date*/ b, falseAnswer) {
	if ( !a && !b ) {
		//both null
		return true;
	}
	else if ( (a && !b) || (!a && b) ) {
		//one is null, the other one isn't
		return falseAnswer;
	}
	else {
		return ( a.getFullYear() == b.getFullYear() &&
			 a.getMonth() == b.getMonth() &&
			 a.getDate() == b.getDate() );
	}
};

egl.timeEquals = function(/*time*/ a, /*time*/ b, falseAnswer) {
	if ( !a && !b ) {
		//both null
		return true;
	}
	else if ( (a && !b) || (!a && b) ) {
		//one is null, the other one isn't
		return falseAnswer;
	}
	else {
		return ( a.getHours() == b.getHours() 
			&& a.getMinutes() == b.getMinutes()
			&& a.getSeconds() == b.getSeconds() );
	}
};

egl.timeStampEquals = function(/*timestamp*/ a, /*timestamp*/ b, falseAnswer) {
	if ( !a && !b ) {
		//both null
		return true;
	}
	else if ( (a && !b) || (!a && b) ) {
		//one is null, the other one isn't
		return falseAnswer;
	}
	else {
		return egl.dateEquals( a, b, falseAnswer ) 
				&& egl.timeEquals( a, b, falseAnswer )
				&& a.getMilliseconds() == b.getMilliseconds();
	}
};

// Constructor for a Delegate (function reference)
// This encapsulates the function and its context in a wrapper function for
// easy invocation
(function() {
	var pkg = egl.makePackage( 'egl.jsrt' );
	if (pkg['Delegate']) { return; }
	
	pkg['Delegate']=function(context,func,name) {
	
		// Invoke the wrapped function in the proper context, with the supplied
		// arguments.
		var d=function() {
			try {
				return func.apply(context, arguments);
			}
			catch (e) {
				if ( e instanceof egl.egl.core.AnyException || (egl.egl.debug && e instanceof egl.egl.debug.DebugTermination))
				{
					throw e;
				}
				throw egl.createRuntimeException( "CRRUI2002E", [ context.eze$$typename, egl.lastFunctionEntered, e.message || e ] );				
			}
		};
		
		// Embed some information in the wrapper for identification purposes
		d.f=func;
		if (name) d.n=name;
		
		// Delegates pointing at the same function are equal
		d.eq=function(o) {
			return this.f === o.f;
		}
		
		// add methods for debug
		d.toString=function() {
		  return this.n;
		}
		
		d.eze$$getChildVariables=function() {
			return [];	
		}
		
		return d;
	};
})();

 egl.defineClass(
	'egl.ui.rui', 'RUILib',
{
	"constructor" : function() {
		if (!egl.egl.ui.rui.RUILib.$inst) {
			egl.egl.ui.rui.RUILib.$inst = this;
		}
		this.invokeListeners = [];
		this.responseListeners = [];
		return egl.egl.ui.rui.RUILib.$inst;
	},
	"sort" : function(array, sortFunction) {
		if (egl.debug) egl.enter("Runtime.sort");
		try {
			array.sort(sortFunction);
		}
		catch (e) {
			throw egl.createRuntimeException( "CRRUI2102E", [] );
		}
		if (egl.debug) egl.leave("");
	},
	"eze$$getName": function() {
		return "egl.ui.rui.RUILib";
	},
	"eze$$getChildVariables": function() {
		return [];
	},
	"setTextSelectionEnabled": function(value) {
		egl.setEnableTextSelection(value);
	},
	"getTextSelectionEnabled": function() {
		return egl.enableSelection;
	},
	"getUserAgent": function(){
		return window.navigator.userAgent;
	},
	"eze$$getChildVariables": function() {
		return [];	
	}
});

egl.createDictionary = function( c, o ) {
	return new egl.egl.Dictionary( c, o );
}

egl.defineClass("egl", "Dictionary", {
	"constructor" : function() {
		this.eze$$caseSensitive = arguments[0] || false;
		this.eze$$byKeyOrdering = arguments[1] || false;
		this.eze$$typename = "Dictionary";
		this.eze$$signature = "y;";
		this.toString = function() {
			return this.$text || "[Dictionary]";
		}
		this.eze$$fromJson = function( object ) {
			   egl.egl.core.$ServiceLib.convertObjectToDictionary( object, this);
	    }
		this.eze$$getChildVariables = function() {
			var children = []; 
			for ( i in this ) {
				if ( typeof this[i] == "function" ) continue;
				if ( i.toString().indexOf("eze$$") == 0 ) continue;
				children.push( {name : i.toString(), value : this[i], type : "any"} );
			}
			return children;
		}
	}
	,
	"eze$$toXML": function( namespaces, xmlName ) {
		var s = ["<" + xmlName];
		s.push(">");
		for (f in this) {
			if (!f.match(/^eze\$\$/) && (typeof this[f] != "function")) {
				s.push( egl.toXMLString( namespaces, f, egl.unboxAny(this[f]) ) );
			}
		}
		s.push("</" + xmlName + ">");
		return s.join('');
	}
	,
	"eze$$fromXML": function( element, xmlName ) {
		var children = egl.egl.core.$XMLLib.getChildElements( element );
		for (var n=0; n < children.length; n++) {
			var value = this.eze$$getChildObjects(children[n]);
			var pairsArray = children[ n ][0].tagName.split(':'); // IE doesn't support localName
			var localName = pairsArray[pairsArray.length-1];
			pairsArray = egl.inferSignature(value).split('?');
			var sig = pairsArray[pairsArray.length-1];
			egl.valueByKey(this,localName,value,sig);
		}
	}
	,
	"eze$$getChildObjects": function( nodelist ) {
		var value = "";
		if (nodelist.length > 1) {
			value = [];
			for (var n=0; n<nodelist.length;n++) {
				value.push(this.eze$$getChildObjects([nodelist[n]]));
			}
		}
		else {
			var children = egl.egl.core.$XMLLib.getChildElements( nodelist[0] );
			if (children.length > 0) {
				value = egl.createDictionary(this.eze$$caseSensitive, this.eze$$byKeyOrdering);
				value.eze$$fromXML(nodelist[0], "");
			}
			else {
				value = egl.egl.core.$XMLLib.getElementTyped( nodelist[0] );
			}
		}
		return value;
	}
});

egl.valueByKey = function egl_valueByKey( object, key, value, signature ) 
{
	if (object == null) {
		return null;
	}
	if ( "eze$$value" in object )
	{
		object = object.eze$$value;
	}
	if (object == null) {
		return null;
	}
	var objectIsRecord = object instanceof egl.egl.jsrt.Record;
	var objectIsDictionary = object instanceof egl.egl.Dictionary;
	if (!(objectIsRecord || objectIsDictionary)) {
		throw egl.createRuntimeException( "CRRUI2024E", [key,egl.inferSignature(object)] );
	}
	
	var result = egl.findByKey(object, key);
	if ( result !== null && ( typeof result !== "object" || !("eze$$value" in result) ) )
	{
		var sig = egl.inferSignature(result);
		if (objectIsRecord && "eze$$getFieldSignatures" in object) {
			var fields = object.eze$$getFieldSignatures();
			var lKey = key.toLowerCase();
			for (var i=0; i<fields.length; i++) {
				var lField = fields[i].name.toLowerCase();
				if (lKey === lField) {
					sig = fields[i].type;
					break;
				}
			}
		}
		result = egl.boxAny( result, sig );
	}
	if (result === null) {
		if ((objectIsRecord && !egl.findKey(object, key))|| (objectIsDictionary && value === undefined)) {
			throw egl.createRuntimeException( "CRRUI2025E", [key, egl.inferSignature(object)] );
		}
	}
	if (value != undefined) 
	{
		if ( signature.charAt(0) === 'A' )
		{
			if (value.eze$$value instanceof egl.egl.jsrt.Record)
			{
				value = egl.boxAny(value.eze$$value.eze$$clone(), value.eze$$signature);
			}
		}
		else
		{
			if (value instanceof egl.egl.jsrt.Record)
			{
				value = value.eze$$clone();
			}
			value = egl.boxAny( value, signature );
		}
		if (objectIsRecord)
		{
			var sig;
			var fields = object.eze$$getFieldSignatures();
			var lKey = key.toLowerCase();
			for (var i=0; i<fields.length; i++) {
				var lField = fields[i].name.toLowerCase();
				if (lKey === lField) {
					sig = fields[i].type;
					break;
				}
			}
			value = egl.convertAnyToType(value,sig);
		}
		if ( objectIsDictionary && object.eze$$caseSensitive==false )
			key = key.toLowerCase();
		object[key] = value;
		result = object[key];
	}
	if (egl.traceInternals && (!result && value == undefined)) {
		egl.addTraceEvent(
			"<div style='border: solid orange 4px; padding:4; color:orange'>"+
			"<b>WARNING</b>: " + 
			"field '"+key+"' not found.<hr>" +
			"<font color=orange>object="+
				egl.toJSONString(object)+"</font><br></div>");
	}
	return result;
};

egl.containsKey = function egl_containsKey( object, key ) 
{	
	var caseSensitive = true;
	if ( (object.eze$$caseSensitive != undefined) && object.eze$$caseSensitive==false )
	{
		caseSensitive = false;
		key = key.toLowerCase();
	}
	for (f in object) {
		var field = (caseSensitive)?f:f.toLowerCase();
		if (key == field) {
			return true;
		}
	}
	return false;
};

egl.findByKey = function egl_findByKey( object, key ) {
	egl.checkWork();
	try {
		var caseSensitive = true;
		if ( (object.eze$$caseSensitive != undefined) && object.eze$$caseSensitive==false )
		{
			caseSensitive = false;
			key = key.toLowerCase();
		}
		for (f in object) {
			var field = (caseSensitive)?f:f.toLowerCase();
			if (key == field) {
				return object[f];
			}
		}
	}
	catch (e) {
		throw egl.createRuntimeException( "CRRUI2103E", [ key, typeof(object), object, e.message] );
	}
	return null;
};

egl.findKey = function egl_findKey( object, key ) {
	egl.checkWork();
	try {
		var caseSensitive = true;
		if ( (object.eze$$caseSensitive != undefined) && object.eze$$caseSensitive==false )
		{
			caseSensitive = false;
			key = key.toLowerCase();
		}
		for (f in object) {
			var field = (caseSensitive)?f:f.toLowerCase();
			if (key == field) {
				return true;
			}
		}
	}
	catch (e) {
		throw egl.createRuntimeException( "CRRUI2103E", [ key, typeof(object), object, e.message] );
	}
	return false;
};

egl.getKeys = function egl_getKeys(object) {
	egl.checkWork();
	var result = [];
	for (field in object) {
		if (egl.isUserField(object, field)) {
			result.push(field);
		}
	}
	if ( (object.eze$$byKeyOrdering != undefined) && 
		 (object.eze$$byKeyOrdering == true) )
		return result.sort();
	else 
		return result;
};

egl.getValues = function egl_getValues(object) {
	var keys = egl.getKeys(object);
	var result = [];
	for (var n=0; n<keys.length; n++) {
		result.push(object[keys[n]]);
	}
	return result;
};

egl.clone = function egl_clone(source) {

	if ( source === null || typeof(source)!="object") {
		return source;
	}
	
	var target;
	if ( source instanceof Array ) {
		target = [];
	}
	else if (source.eze$$thisClass != undefined){
		target = new source.eze$$thisClass();
	}
	else {
		target = {};
	}
	
	for (f in source) {
		if (typeof(source[f])=="object") {
			target[f] = egl.clone(source[f]);
		}
		else {
			target[f] = source[f];
		}
	}
	return target;
}

egl.insert = function egl_insert(target, key, value) {
	if (typeof(value)=="object") {
		target[key] = egl.clone(value);
	}
	else {
		return (target[key] = value);
	}
};

egl.insertAll = function egl_insertAll(target, source) {
	for (f in source) {
		if (egl.isUserField(source,f)) {
			target[f] = source[f];
		}
	}
};

egl.removeElement = function egl_removeElement(object, key) {
	key = object.eze$$caseSensitive ? key : key.toLowerCase() ;
	for (f in object) {
		var field = (object.eze$$caseSensitive)?f:f.toLowerCase();
		if (key == field) {
			delete object[f];
			return;
		}
	}
};

egl.removeAll = function egl_removeAll(object) {
	for (field in object) {
		if ( egl.isUserField(object, field) ) {
			delete object[field];
		}
	}
};

egl.size = function egl_size(object) {
	var count = 0;
	for (field in object) {
		if (egl.isUserField(object, field))   
			count++;
	}
	return count;
};

egl.internalFieldMatcher = /eze\$\$/;

egl.isUserField = function(object, field) {
	return typeof(object[field]) != "function" && 
			!field.match(/^[0-9]*$/) && 
			!field.match(egl.internalFieldMatcher);
}
	
String.prototype.splice = function( first, last, instr ) {
	var result = this.substring(0, first-1);
	for (var n=first; n<=last && n<this.length; n++)
		result += instr.charAt(n-first);
	result += this.substring(last);
	return result;
};

egl.isspace = function(c) { 
	return c.match(/\s/); 
};
egl.isblanks = function(s) {
	if (s == null) return true;
	s = egl.trim(s);
	return s.length==0;
};
egl.isnumeric = function(s) {
	if (s == null) return true;
	return (s.length!=0 && s.match(/[^0-9]+/)==null);
};

egl.trim = function(s) {
	return s.replace(/^\s+|\s+$/g,"");
};
	
String.prototype.trim = function() {
	var n = this.length;
	while (egl.isspace(this.charAt(n-1))) 
		--n;
	return this.substring(0,n);
};
	
String.prototype.cmp = function(s2,eq) {
	s2 = (s2 && s2.toString()) || "";
	if (eq) return (this==s2) ? 0 : 1;
	return (this<s2) ? -1 : (this>s2) ? 1 : 0;
}
	
String.prototype.eq = function(s) {
	return this.cmp(s,true) == 0;
};
	
String.prototype.ne = function(s) {
	return this.cmp(s,true) != 0;
};
	
String.prototype.gt = function(s) {
	return this.cmp(s) > 0;
};
	
String.prototype.ge = function(s) {
	return this.cmp(s) >= 0;
};

String.prototype.lt = function(s) {
	return this.cmp(s) < 0;
};
	
String.prototype.le = function(s) {
	return this.cmp(s) <= 0;
};

egl.javascriptRegExpAlias = function(ch) {
	//takes in a single character and returns an appropriate form to use in regular expressions
	//certain symbols hold special meaning, but calling this function will add a \ to treat them
	//like any character
	switch (ch) {
	case "^":
	case "$":
	case "*":
	case "+":
	case "?":
	case ".":
	case "(":
	case ")":
	case "[":
	case "]":
	case "{":
	case "}":
	case "\\":
		return "\\" + ch;
	
	//nuthin' fancy
	default:
		return ch;
	}
}
	
String.prototype.like = function(pattern, escape) {	
	//EGL 'like' is similar to SQL regular expressions.
	//Change % to .* and _ to . so that we can use RegExp
	var trimP = pattern.trim();
	var newPattern = "";
	
	for ( var i = 0; i < trimP.length; i++ ) {
		switch ( trimP.charAt(i) ) {
		
		case "%":
			newPattern += ".*";
			break;
			
		case "_":
			newPattern += ".";
			break;
			
		//escape character found
		//\% -> %, \_ -> _, but for anything else, the escape character is ignored
		case escape:
			var tempChar = trimP.charAt(++i);
			//extra handling if the escape char is backslash
			//can filter universal escape chars through to the regexp (\t, \n, etc)
			if ( escape == "\\" && tempChar != "%" && tempChar != "_" ) {
				newPattern += "\\";
			}
			newPattern += egl.javascriptRegExpAlias( tempChar );
			break;
			
		//regular character, but make it safe for use in a regexp
		default:
			newPattern += egl.javascriptRegExpAlias( trimP.charAt(i) );
			break;	
		}
	}
	
	return (new RegExp( newPattern )).test( this );		
};
	
String.prototype.matches = function(pattern, escape) {
	//converts EGL * ? [^-] symbols to javascript equivalents for matches
	var newPattern = "";
	
	for ( var i = 0; i < pattern.length; i++ ) {
		switch ( pattern.charAt(i) ) {
		
		case "*":
			newPattern += ".*";
			break;
			
		case "?":
			newPattern += ".";
			break;
			
		//escape character found
		case escape:
			var tempChar = pattern.charAt(++i);
			//extra handling if the escape char is backslash
			//can filter universal escape chars through to the regexp (\t, \n, etc)
			if ( escape == "\\" && tempChar != "*" && tempChar != "?" ) {
				newPattern += "\\";
			}
			newPattern += egl.javascriptRegExpAlias( tempChar );
			break;
			
		case "[":
			//find a matching ]
			var tempi = i + 1;
			while ( tempi < pattern.length && pattern.charAt(tempi) != "]" ) {
				tempi++;
			}
			
			if ( tempi == pattern.length || pattern.charAt(tempi - 1) == escape ) {
				//no matching ] found
				//needed to also make sure \] doesn't count as ]
				newPattern += "\\[";
			}
			else {
				//matching ] found
				//add everything between the brackets (at the risk of the user)
				newPattern += pattern.substring(i, tempi + 1 );
				i = tempi;
			}
			break;
			
		//regular character, but make it safe for regexp
		default:
			newPattern += egl.javascriptRegExpAlias( pattern.charAt(i) );
			break;	
		}
	}
	
	return (new RegExp( newPattern )).test( this );
};

egl.defineClass( "egl.jsrt", "Record", {
	"constructor" : function() {
		if (arguments.length > 0) 
			this.assign(arguments[0]);
		else
			this.eze$$fieldNames = [];
		this.eze$$isNull = false;
		this.eze$$isNullable = false;
		this.eze$$caseSensitive = false;
	},
	
	"eze$$setEmpty" : function() {
	},
	
	"eze$$clone" : function() {
	},
	
	"setNull" : function(bNull) {
		if (this.eze$$isNullable){
			this.eze$$isNull = Boolean(bNull);
		}
		else if (Boolean(bNull)) {
			this.eze$$setEmpty();
		}
	},
	
	"isNull" : function(bNull) {
		return (this.eze$$isNullable && this.eze$$isNull);
	},
	
	"assign" : function(array) {
		for (var n=0; n<array.length; n++) {
			this[array[n][0]] = array[n][1];
		}
	},
	
	"getValues" : function() {
		var ret = [];
		for (var f in this) {
			if ( typeof this[f] == "function" ) continue;
			if ( !this.propertyIsEnumerable(f) ) continue;
			ret.appendElement( this[f] );
		}
		return ret;
	},

	/*
	 * Return the signature (Java Style) for this object
	 */
	"signature" : function() {
		var returnValue = "T";
		if ( this.eze$$package.length>0 ) returnValue += (this.eze$$package.replace( /\./g, "/" ) + "/");
		returnValue += ( this.eze$$typename + ";" );
		return returnValue;
	}
	,
	"eze$$getFieldSignatures": function() {
		return [];
	}
	,
	
	"eze$$getChildVariables" : function() {
		if (!(this.eze$$fieldNames)) {
			return null;
		}
		
		var childVars = [];
		for (var i=0; i<this.eze$$fieldNames.length; i++) {
			childVars.push({name : this.eze$$fieldNames[i], value : this[this.eze$$fieldNames[i]]});			
		}
		
		return childVars;
	}
	,
	"eze$$getJSONName": function(/*string*/ fieldName) {
		var jsonNames = this.eze$$getJSONNames();
		for (var n=0; n<jsonNames.length; n++) {
			if (jsonNames[n][0] == fieldName)
			{
				return jsonNames[n][1];
			}
		}
		return fieldName;
	}
	,
	"eze$$getFORMName": function(/*string*/ fieldName) {
		var formDataNames = this.eze$$getFORMDataNames();
		for (var n=0; n<formDataNames.length; n++) {
			if (formDataNames[n][0] == fieldName)
			{
				return formDataNames[n][1];
			}
		}
		return fieldName;
	}
	,
	"eze$$getFORMDataNames": function() {
		return [];
	}
	,
	"eze$$getJSONNames": function() {
		return [];
	}
});

egl.formatMessage = function( msg, inserts )
{
	var formattedText = msg;
	if ( inserts != null )
	{
		for ( i = 0; i < inserts.length ; i++ )
		{
			var pattern = "{" + i + "}";
			formattedText = formattedText.replace( pattern, inserts[i] );
		}
	}
	return formattedText;
}

egl.getRuntimeMessage = function( rscKey, inserts ) 	
{
	var rscName = "RuiMessages";
	var rscString = rscKey;
	
	if ( ( rscName != null ) && ( egl.eze$$rscBundles[rscName] != undefined ) )
	{
		var rscBundle = egl.eze$$rscBundles[rscName];
		if ( (rscKey!=null) && (rscBundle[rscKey] != null) )
		{
			rscString = rscBundle[rscKey];
			rscString = egl.formatMessage( rscString, inserts );
		}
	}
	if ( rscKey != undefined && rscString == rscKey )
	{
		for ( f in egl.eze$$rscBundles )
		{
			var rscBundle = egl.eze$$rscBundles[ f ];
			if (( f != "RuiMessages") && (rscKey!=null) && (rscBundle[rscKey] != null) )
			{
				rscString = rscBundle[rscKey];
				rscString = egl.formatMessage( rscString, inserts );
				break;
			}
		}
	}
	if ( rscString != rscKey )
	{
		rscString = "[" + rscKey + "] " + rscString; 
	}
	return rscString;
}

egl.defineClass(
		'egl.ui.rui', 'RUIPropertiesLibrary',
		{
		"constructor" : function() {
			this.normalizedKeys = {};
		},
		"eze$$initializeProperties" : function(/*String*/ propertiesFile){
			var bundle = egl.eze$$rscBundles[propertiesFile];
			if(bundle == null){
				// Missing bundle
				var path = 'properties/' + propertiesFile + '-' + egl__htmlLocale + '.properties';
				throw egl.createRuntimeException( 'CRRUI2015E', [ path, propertiesFile ] );
			}
			// Every RUIPropertiesFile has a getProperties function, which returns an array of the fields
			// defined in the library
			var keys = this.eze$$getProperties();
			for ( var i = 0; i < keys.length; i++ ) {
				var key = keys[i];
				var value = bundle[key];
				if(value == null){
					// Missing property, set the value to the key
					this[key] = key;
				}else{
					this[key] = value;
				}
				// Store a normalized link back to the original key
				this.normalizedKeys[key.toLowerCase()] = key;
			}
		},	
		"eze$$getValueWithNormalizedKey" : function(/*String*/ key){
			var value = this[this.normalizedKeys[ key.toLowerCase() ]];
			if(value == null){
				value = "";
			}
			return value;
		},
		"getMessage" : function(/*String*/ key, /*String[]*/ inserts){
			return egl.formatMessage(key, inserts);
		},
		"eze$$getChildVariables": function() {
			return [];	
		}
});

Array.prototype.checkIndex = function Array_checkIndex( idx )
{
	if ( ( idx >= this.length ) || ( idx < 0 ) )
	{
		throw egl.createIndexOutOfBoundsException( 'CRRUI2022E', [ idx + 1, this.length ], idx + 1 );
	}
	else
		return idx;
}

egl.checkNull = function ( obj, source )
{
	if ( obj == null )
	{
		if ( source )
			throw egl.createNullValueException( "CRRUI2023E", [ source ] );
		else
			throw egl.createNullValueException( "CRRUI2005E", [] );
	}
	else
		return obj;
}

egl.makeExceptionFromCaughtObject = function( caught )
{
	var message;
	var name;	
	if ( caught instanceof Error )
	{
		name = caught.name;
		message = caught.message;
	}
	else 
	{
		name = "";
		message = String( caught );
	}

	return egl.createJavaScriptObjectException( 'CRRUI2006E', message, name );
}
