/*
 * Licensed Materials - Property of IBM
 *
 * Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA DP Schedule Contract with IBM Corp.
 */
 
 
egl.formatNONE = 1;
egl.formatXML= 2;
egl.formatJSON = 3;
egl.formatFORM = 4;

//const strings
egl.STATEFULSESSIONID = "egl_statefulsessionid";
egl.HEADER_EGLPRIVATE = "EGLPRIVATE";
egl.HEADER_EGLREST = "EGLREST";
egl.HEADER_EGLSOAP = "EGLSOAP";
egl.SERVICE_TIMEOUT = "EGL_TIMEOUT";
    
egl.defineClass(
    'egl.jsrt', 'ServiceRefWrapper',
{
    //these could be in the base class
    //String f_type;			//type of service, "web" or "rest"
    //String f_bindingName
    //String f_RESTmethod;
    //int f_requestFormat;
    //int f_responseFormat;
    //function f_httpCallback;
    //function f_errCallback;    
    //Dictionary f_RESTRequestHeader;
    
    //Array of Object f_callbackArgs;

    "constructor" : function(/*String*/ name) {     
    	this.f_type;     
    	this.f_bindingName = name || "";
        this.f_RESTmethod;
        this.f_requestFormat;
        this.f_responseFormat;  
        this.f_httpCallback = function(){};
        this.f_errCallback = function(){};
        this.f_RESTRequestHeader = null;
        this.f_callbackArgs = {};
    },   
    "getRESTRequestHeader" : function(){
        if(!this.f_RESTRequestHeader || this.f_RESTRequestHeader == null)           
            this.f_RESTRequestHeader = egl.createDictionary(true, false);
        return  this.f_RESTRequestHeader;
    },
    "setServiceTimeoutInHeader" : function(/*int*/ serviceTimeout){
        if( serviceTimeout > -1 )
        	egl.valueByKey( this.getRESTRequestHeader(), egl.SERVICE_TIMEOUT, serviceTimeout, "I;");
        else
        	egl.removeElement(this.getRESTRequestHeader(), egl.SERVICE_TIMEOUT);
    },
    "invokeService" : function(/*RUIHandler*/ handler,
					            /*func*/ httpCallback, 
					            /*func*/ errorCallback, 
					            /*int*/ serviceTimeout,			            
					            /*String*/ functionName,						            
					            /*Array of Objects*/ inParamVals,
					            /*Array of signature strings*/ inParamSignatures,
					            /*String*/ paramOrders,
					            /*Array of Functions or Array*/ callbackArgs){
		throw egl.createRuntimeException( "CRRUI3661E", [this.f_bindingName,functionName] );
    }
});
  
egl.defineClass(
    'egl.jsrt', 'RESTServiceRefWrapper',
    'egl.jsrt', 'ServiceRefWrapper',
{        
    //String f_baseURI;   
    //String f_statefulSessionIdName;
    //String f_uriTemplate;
    
    "constructor" : function(/*String*/ name, /*String*/ baseURI, /*String*/ eglStatefulSessionIdName) {
    	this.f_bindingName = name || "";
        this.f_baseURI = baseURI || "";
        this.f_statefulSessionIdName = eglStatefulSessionIdName || "JSESSIONID";
        this.f_uriTemplate="";    
        this.f_type = "rest";
        this.f_setSessionCookie = false;		//only used for EGL JSON-RPC service call
    },    
        
    "invokeService" : function(/*RUIHandler*/ handler,
					            /*func*/ httpCallback, 
					            /*func*/ errorCallback, 
					            /*int*/ serviceTimeout,
					            /*String*/ functionName,						            
					            /*Array of Objects*/ inParamVals,
					            /*Array of signature strings*/ inParamSignatures,
					            /*String*/ paramOrders,
					            /*Array of Functions or Array*/ callbackArgs, 
					            
			//the following parameters are only used for Rest service call
					            /*boolean*/ hasXXXRestAnnotation,
					            /*String*/ resolvedUriTemplate,
					            /*int*/ requestFormat,
					            /*int*/ responseFormat,
					            /*String*/ restMethod,                        
					            /*String, Dictionary, Record or XMLElement*/ resourceParamIn){
    	
    	this.setServiceTimeoutInHeader(serviceTimeout);
    	if(hasXXXRestAnnotation == true){
    		this.invokeRESTService(handler, 
				    				httpCallback, 
				    				errorCallback,
				    				resolvedUriTemplate,
				    				requestFormat, responseFormat, restMethod,
				    				resourceParamIn, 
				    				callbackArgs);
    	}
    	else{
    	    this.invokeEGLService(handler, 
    	    						httpCallback, 
    	    						errorCallback, 
    	    						functionName,
    	    						inParamVals,
    	    						inParamSignatures,
    	    						callbackArgs);    		
    	}
    },
    /**
     * handler is the RUIHandler, is used as context for the callback function 
     * if GET, parameters is used as queryParameter 
     * if not GET, parameters is used as the resource parameter   
     */
    "invokeRESTService" : function(
                        /*RUIHandler*/ handler,
                        /*func*/ httpCallback, 
                        /*func*/ errorCallback, 
                        /*String*/ resolvedUriTemplate,
                        /*int*/ requestFormat,
                        /*int*/ responseFormat,
                        /*String*/ restMethod,                        
                        /*String, Dictionary, Record or XMLElement*/ resourceParamIn,
                        /*Array of Functions or Array*/ callbackArgs){
        this.f_httpCallback = httpCallback;
        this.f_errCallback = errorCallback;       
        this.f_callbackArgs = callbackArgs;
        
        this.f_RESTmethod = restMethod || "GET";
        this.f_uriTemplate = resolvedUriTemplate || "";
        this.f_requestFormat = requestFormat || egl.formatNONE;
        this.f_responseFormat = responseFormat || egl.formatNONE;        
                                    
        var url = this.f_baseURI + this.f_uriTemplate;
        
        if(this.f_RESTmethod == "GET" || this.f_RESTmethod == "DELETE"){
            egl.egl.jsrt.$ServiceRT.internalInvokeService(
                        handler,
                        url, 
                        this, 
                        resourceParamIn,
                        "");                    //when GET, body is empty string
        }
        else{
            var body = "";        
            //need to build the body using resourceParamIn            
            if(resourceParamIn){
                //check for the requestFormat
                switch(this.f_requestFormat)
                {
                case egl.formatNONE:
                    body = resourceParamIn;
                    break;
                case egl.formatJSON:
                    body = egl.egl.core.$ServiceLib.convertToJSON(resourceParamIn);
                    break;
                case egl.formatXML:   
                    body = egl.egl.core.$XMLLib.convertToXML(resourceParamIn);
                    break;
                case egl.formatFORM:
                    body = egl.egl.core.$ServiceLib.convertToFormData(resourceParamIn);                    
                    break;
                }
            }
            
            egl.egl.jsrt.$ServiceRT.internalInvokeService(
                        handler,
                        url, 
                        this, 
                        {},                     //pass null for query resourceParamIn 
                        body);
        }
        
    },
    
    "buildURL": function(/*String*/ baseURI, /*String*/ uriTemplate){
        var url = baseURI + uriTemplate;
        if(baseURI.length > 0 && uriTemplate.length > 0){
            var end = baseURI.charAt(baseURI.length-1);
            var begin = uriTemplate.charAt(0);

            if(end == '/' && begin == '/'){
                url = baseURI + uriTemplate.substr(1);
            }
        }
        return url;
    },
    /**
     * handler is the RUIHandler, is used as context for the callback function 
     * inParamVals is a array of ordered in and inout parameter values
     * the request and response uses json-rpc
     */
    "invokeEGLService" : function(
                        /*RUIHandler*/ handler,
                        /*func*/ httpCallback, 
                        /*func*/ errorCallback, 
                        /*String*/ functionName,  
                        /*Array of Objects*/ inParamVals,
                        /*Array of signature strings*/ inParamSignatures,
                        /*Array of Functions or Array*/ callbackArgs){

        egl.valueByKey( this.getRESTRequestHeader(), egl.HEADER_EGLREST, "TRUE", "S;" );  
        
        //set the session id for stateful calls
        egl.valueByKey( this.getRESTRequestHeader(), egl.STATEFULSESSIONID, this.f_statefulSessionIdName, "S;" );                
        var sessionIdObj = egl.findByKey(egl.statefulSessionMap, this.f_baseURI);
        if(sessionIdObj != null){
        	var sessionIdValue = sessionIdObj.sidVal;
        	
        	var cookieValue = egl.findByKey(this.getRESTRequestHeader(), "Cookie");
        	if(cookieValue != null){	
	        	cookieValue = cookieValue.eze$$value;
        	}
        	else
        		cookieValue = "";
        	
        	if(!this.f_setSessionCookie){        		
        		if(cookieValue != ""){
               		//if there already exists a cookie, append to it
            		cookieValue += "; ";
        		}        			        			
            	var newCookieValue = cookieValue + sessionIdValue;        		
         		
            	egl.valueByKey( this.getRESTRequestHeader(), "Cookie", newCookieValue, "S;");
            	this.f_setSessionCookie = true;        		
        	}
        	else{
        		if(sessionIdObj.oldVal != null && sessionIdObj.oldVal != undefined){
        			//need to parse the existing cookieValue, replace with the new value        			
        			var toBeReplaced = new RegExp(sessionIdObj.oldVal, "i");        			
        			cookieValue = cookieValue.replace(toBeReplaced, sessionIdValue);
        			
                	egl.valueByKey( this.getRESTRequestHeader(), "Cookie", cookieValue, "S;");
                	this.f_setSessionCookie = true;        		        			
        		}
        		//else if sessionIdObj.oldVal == null && this.f_setSessionCookie == true, 
        		//the session cookie is already set in the header, do nothing 
        	}
        	
        }        
        
        //uses json-rpc     
        this.f_RESTmethod = 'POST';
        this.f_requestFormat = egl.formatJSON;
        this.f_responseFormat = egl.formatJSON;                     
                        
        this.f_httpCallback = httpCallback;
        this.f_errCallback = errorCallback;
        this.f_callbackArgs = callbackArgs;
        var url = this.f_baseURI;
        
        //let's build the payload body using json-rpc
        var payloadbodyObj = {
            bindingName: this.f_bindingName,
            method: functionName || "",
            params: inParamVals || [],
            eze$$InParamSignatures: inParamSignatures || []
        };
        
        var body = egl.egl.core.$ServiceLib.convertToJSON(payloadbodyObj);        
        egl.egl.jsrt.$ServiceRT.internalInvokeService(
                    handler,
                    url,                     //no uri
                    this, 
                    {},                     //pass null for query parameters 
                    body);        
    }    
    
});

egl.defineClass(
	    'egl.jsrt', 'EGLPrivateServiceRefWrapper',
	    'egl.jsrt', 'ServiceRefWrapper',
{        
	    //String f_serviceName;   
	    //String f_alias;   
    
    "constructor" : function(/*String*/ variableName, /*String*/ serviceName, /*String*/ alias) {
	        	this.f_bindingName = variableName || "";
	        	this.f_serviceName = serviceName || "";
	        	this.f_alias = alias || "";
	        	this.f_type = "egl";
    },    
        
    "invokeService" : function(/*RUIHandler*/ handler,
					            /*func*/ httpCallback, 
					            /*func*/ errorCallback, 
					            /*int*/ serviceTimeout,
					            /*String*/ functionName,						            
					            /*Array of Objects*/ inParamVals,
					            /*Array of signature strings*/ inParamSignatures,
					            /*String*/ paramOrders,
					            /*Array of Functions or Array*/ callbackArgs){
    	
    	this.setServiceTimeoutInHeader(serviceTimeout);
	    this.invokeEGLService(handler, 
	    						httpCallback, 
	    						errorCallback, 
	    						functionName,
	    						inParamVals,
	    						inParamSignatures,
	    						callbackArgs);    		
    },
    /**
     * handler is the RUIHandler, is used as context for the callback function 
     * inParamVals is a array of ordered in and inout parameter values
     * the request and response uses json-rpc
     */
    "invokeEGLService" : function(
                        /*RUIHandler*/ handler,
                        /*func*/ httpCallback, 
                        /*func*/ errorCallback, 
                        /*String*/ functionName,  
                        /*Array of Objects*/ inParamVals,
                        /*Array of signature strings*/ inParamSignatures,
                        /*Array of Functions or Array*/ callbackArgs){

        egl.valueByKey( this.getRESTRequestHeader(), egl.HEADER_EGLPRIVATE, "TRUE", "S;" );  
        
        
        //uses json-rpc     
        this.f_RESTmethod = 'POST';
        this.f_requestFormat = egl.formatJSON;
        this.f_responseFormat = egl.formatJSON;                     
                        
        this.f_httpCallback = httpCallback;
        this.f_errCallback = errorCallback;
        this.f_callbackArgs = callbackArgs;
        var url = "";
        
        //let's build the payload body using json-rpc
        var payloadbodyObj = {
            bindingName: this.f_bindingName,
            method: functionName || "",
            params: inParamVals || [],
            eze$$InParamSignatures: inParamSignatures || [],
            binding:{
        		name: this.f_bindingName,
                type: "EGLBinding",
                serviceName: this.f_serviceName,
                alias: this.f_alias,
                protocol: "local"
           }
        };
        
        var body = egl.egl.core.$ServiceLib.convertToJSON(payloadbodyObj);        
        egl.egl.jsrt.$ServiceRT.internalInvokeService(
                    handler,
                    url,                     //no uri
                    this, 
                    {},                     //pass null for query parameters 
                    body);        
    }    
    
});

egl.defineClass(
    'egl.jsrt', 'SOAPServiceRefWrapper',
    'egl.jsrt', 'ServiceRefWrapper',
{
    //String f_fqInterface;    
    //String f_wsdlLocation;
    //String f_wsdlService;
    //String f_wsdlPort;
    //String f_wsdlUri;
    
    "constructor" : function(/*String*/ FQInterfaceName,
    						 /*String*/ simpleInterfaceName,
                             /*String*/ wsdlLocation, 
                             /*String*/ wsdlService,
                             /*String*/ wsdlPort,
                             /*String*/ uri) {        
        this.f_fqInterface = FQInterfaceName || "";
        this.f_bindingName = simpleInterfaceName || "";
        this.f_wsdlLocation = wsdlLocation || "";
        this.f_wsdlService = wsdlService || "";
        this.f_wsdlPort = wsdlPort || "";
        this.f_wsdlUri = uri || "";
        this.f_type = "web";                 	
        
        this.initOtherMemebers();
    },
        
    "initOtherMemebers" : function(){
        this.f_RESTmethod = 'POST';
        this.f_requestFormat = egl.formatJSON;
        this.f_responseFormat = egl.formatJSON;                    	
    },
        
    "invokeService" : function(/*RUIHandler*/ handler,
					            /*func*/ httpCallback, 
					            /*func*/ errorCallback, 
					            /*int*/ serviceTimeout,			            
					            /*String*/ functionName,						            
					            /*Array of Objects*/ inParamVals,
					            /*Array of signature strings*/ inParamSignatures,
					            /*String*/ paramOrders,
					            /*Array of Functions or Array*/ callbackArgs){
					            
//============================ the following parameters are not used for soap  ============================    	
//					            /*boolean*/ hasXXXRestAnnotation,
//					            /*String*/ resolvedUriTemplate,
//					            /*int*/ requestFormat,
//					            /*int*/ responseFormat,
//					            /*String*/ restMethod,                        
//					            /*String, Dictionary, Record or XMLElement*/ resourceParamIn
//=========================================================================================================    	
    	this.setServiceTimeoutInHeader(serviceTimeout);
    	this.invokeSOAPService(handler, 
	    						httpCallback, 
	    						errorCallback, 
	    						functionName,
	    						inParamVals,
	    						inParamSignatures,
	    						paramOrders,
	    						callbackArgs);    		
    },    
    
    /**
     * handler is the RUIHandler, is used as context for the callback function
     * inParamVals is a array of ordered in and inout parameter values 
     */    
    "invokeSOAPService" : function(
                        /*RUIHandler*/ handler,
                        /*func*/ httpCallback, 
                        /*func*/ errorCallback, 
                        /*String*/ operationName,                        
                        /*Array of Objects*/ inParamVals,
                        /*Array of signature strings*/ inParamSignatures,
                        /*String*/ paramOrders,
                        /*Array of Functions or Array*/ callbackArgs){
        this.f_httpCallback = httpCallback;
        this.f_errCallback = errorCallback;    
        this.f_callbackArgs = callbackArgs;
        
        egl.valueByKey( this.getRESTRequestHeader(), egl.HEADER_EGLSOAP, egl.contextRoot, "S;" );        
        
        //let's build the payload body using json-rpc
        var payloadbodyObj = {
            bindingName: this.f_bindingName,
            method: operationName || "",
            params: inParamVals || [],
            eze$$InParamSignatures: inParamSignatures || [], 
            EGL_PARAMETER_ORDER: paramOrders || "",
            binding:{
                type: "WebBinding",
                interfacename: this.f_fqInterface,
                name: this.f_bindingName,
                wsdlLocation: this.f_wsdlLocation,
                wsdlService: this.f_wsdlService,
                wsdlPort: this.f_wsdlPort,
                uri: this.f_wsdlUri
                }
        };
        
        var body = egl.egl.core.$ServiceLib.convertToJSON(payloadbodyObj);
        egl.egl.jsrt.$ServiceRT.internalInvokeService(
                    handler,
                    "",                     //no uri
                    this, 
                    {},                     //pass null for query parameters 
                    body);
        
    }    
});  

egl.defineClass(
    'egl.jsrt', 'ServiceRT',
{
    "constructor" : function() {
        if (egl.egl.jsrt.$ServiceRT) 
            return egl.egl.jsrt.$ServiceRT;         
        egl.egl.jsrt.$ServiceRT=this;
        
        this.eze$$proxyURL = '___proxy';        
    },
    
    "setRequestContentTypeHeaderIfNotSet" : function(/*ServiceWrapper*/ serviceWrapper){
        //if it already has content-type in the header, then do not do anything
        //otherwise, set the content type based on the request format
        var hasReqContentType = egl.containsKey(serviceWrapper.getRESTRequestHeader(), "Content-Type");
        if(hasReqContentType == false){
            var contentType = "text/html"
            switch(serviceWrapper.f_requestFormat)
            {
            case egl.formatJSON:
                contentType = "application/json";
                break;
            case egl.formatXML:                  
                contentType = "application/xml";   
                break;
            case egl.formatFORM:
                contentType = "application/x-www-form-urlencoded"                    
                break;
            }           
            contentType += "; charset=UTF-8"
            
            egl.valueByKey( serviceWrapper.getRESTRequestHeader(), "Content-Type", contentType, "S;");                    
        }
    },

    "getExceptionSource" : function(/*String*/ serviceCalltype){
    	if(serviceCalltype == "rest"){
    		return egl.core.ServiceKind.REST;
    	}
    	    	
    	return egl.core.ServiceKind.WEB;
    },
    
    "isJsonRPCFormat" : function(/*HttpRequest*/ request){
    	var isJSONRPC = false;
    	if(request.headers){
            //let's check the header of the request to see if it's soap or low egl call
    		//if so, it uses json-rpc as response
    		var isSOAP = egl.containsKey(request.headers, egl.HEADER_EGLSOAP);
    		var isEGLREST = egl.containsKey(request.headers, egl.HEADER_EGLREST);
    		var isEGLPrivate = egl.containsKey(request.headers, egl.HEADER_EGLPRIVATE);
    		if(isSOAP || isEGLREST || isEGLPrivate ){
    			isJSONRPC = true;
    		}
    	}
    	return isJSONRPC;
    },
    
    /**
     * This is an overloaded function with the following patterns:     
            Function internalInvokeService( handler RUIHandler, uri String, serviceWrapper, parameters Dictionary) end 
            Function internalInvokeService( handler RUIHandler, uri String, serviceWrapper, parameters Dictionary, body String) end        
     */              
    "internalInvokeService" : function(/*RUIHandler*/ handler, 
                            /*String*/ uriString,               
                            /*ServiceWrapper*/ serviceWrapper, 
                            /*Dictionary*/ queryParam, 
                            /*String*/ bodyString) {
    	//make a shallow copy of some of the properties on the serviceWrapper, 
    	//these copies will be used in the callback function, this way, even if serviceWrapper object got changed by other event
    	//the values used in the callbacks are accurate 
    	var callbackFunc = serviceWrapper.f_httpCallback;		//shallow copy
    	var errCallbackFunc = serviceWrapper.f_errCallback;		//shallow copy
    	var callbackArgs = serviceWrapper.f_callbackArgs;		//shallow copy
    	var responseFormat = serviceWrapper.f_responseFormat;	//copy by value
    	var serviceCallType = serviceWrapper.f_type;			//copy by value
        var request = null;       
        var requestExp = null;
        try {
            this.setRequestContentTypeHeaderIfNotSet(serviceWrapper);           
            request = { 
                uri: uriString, 
                queryParameters: queryParam || {}, 
                method: serviceWrapper.f_RESTmethod || "GET", 
                body: bodyString || "",
                headers: serviceWrapper.getRESTRequestHeader() || {}
            };
        }
        catch (e) {
    		requestExp = egl.createAnyException("CRRUI3654E", [e]);    		
        }       
        
        if(requestExp != null){
        	egl.egl.jsrt.$ServiceRT.callErrorCallback(errCallbackFunc, uriString, handler, requestExp);
        }
        else{
	        this.doInvokeAsync( 
	            request,
	            serviceCallType,
	            function(response) {
	            	var isJSONRPC = egl.egl.jsrt.$ServiceRT.isJsonRPCFormat(request);
	            	
	            	var anyExp = null;
	                try {                	
	                    if(response.status >= 200 && response.status <= 299){
	                        egl.egl.jsrt.$ServiceRT.setStaticRESTResponse(response);                      	
	                    	var sessionIdValue = egl.findByKey(response.headers, egl.STATEFULSESSIONID);
	                    	if(sessionIdValue != null){
	                    		//try to get the current value
	                    		var currVal = egl.statefulSessionMap[request.uri];
	                    		if(currVal != null && currVal != undefined){
	                    			//compare to see if it's the same as the new one
	                    			if(sessionIdValue != currVal.sidVal){
	                    				currVal.oldVal = currVal.sidVal;
	                    				currVal.sidVal = sessionIdValue;
	                    			}
	                    			else //reset old value to null, since oldVal is only used to indicate the session value has changed
	                    				currVal.oldVal = null;
	                    		}
	                    		else{
	                    			var sessionIdObj = new Object();
	                    			sessionIdObj.sidVal = sessionIdValue;
	                    			sessionIdObj.oldVal = sessionIdValue;
	                    			egl.statefulSessionMap[request.uri] = sessionIdObj;
	                    		}
	                    	}
	                    	
		                    switch(responseFormat)
		                    {
		                    case egl.formatNONE:
		                        if(callbackArgs.length > 0){
		                            callbackArgs[callbackArgs.length-1] = response.body;
		                        }
		                        break;
		                    case egl.formatJSON:
		                        var jsonObj = egl.egl.core.$ServiceLib.convertFromJSON( response.body );
		                        for(var i=0; i<callbackArgs.length; i++){
		                        	//replace each of element in the callbackArgs with the object each of the function returns		                        	
	                            	if(isJSONRPC == true){ //the response body is in json-rpc object, get the result property                            		
	                            		//if there is only one return/out 
	                            		if(callbackArgs.length == 1){
	                            			callbackArgs[i] = egl.egl.jsrt.$ServiceRT.processCallbackArg(callbackArgs[i], jsonObj.result);
	                            		}
	                            		else{ //if(jsonObj.result instanceof Array)  //result must be an array 
	                            			callbackArgs[i] = egl.egl.jsrt.$ServiceRT.processCallbackArg(callbackArgs[i], jsonObj.result[i]);
	                            		}
	                            	}
	                            	else
	                            		callbackArgs[i] = egl.egl.jsrt.$ServiceRT.processCallbackArg(callbackArgs[i], jsonObj);	                        		                        	
		                        }	                        
		                        break;
		                    case egl.formatXML:
		                        if(callbackArgs.length > 0){ //the last parameter is the return record
		                        	//execute the function, pass 1, so the 1st parameter isn't null, it will pass the nullable check, 
		                        	//we're assuming it's never null in xml
		                        	//then return the record
		                        	//replace the element in the callbackArgs with the record it returned
		                        	callbackArgs[callbackArgs.length-1] = callbackArgs[callbackArgs.length-1](1);
		                        	egl.egl.core.$XMLLib.convertFromXML(response.body, callbackArgs[0]);
		                        }
		                        break;
		                    case egl.formatFORM:
		                    	anyExp = egl.createAnyException("CRRUI3656E", []);	                    	
		                        break;
		                    }
		                    
		                    if (anyExp == null && callbackFunc != null){
		                        //if there is a call back function
		                    	if (egl.debugg) {
		                    		egl.debugCallback(handler, callbackFunc, callbackArgs);
		                    	}
		                    	else {
		                    		try {
		                    			callbackFunc.apply(handler, callbackArgs);
		                    		}
		                    		catch (e) {
		                    			var msg = egl.getRuntimeMessage( "CRRUI1070E", [e.message] );
		                    			egl.printError( msg, e);
		                    			//continue throw e
		                    			throw egl.createRuntimeException("CRRUI1070E", [e.message]);
		                    		}
		                    	}
		                    }	                    	                    
	                    }
	                    else{ //inner response is not 2xx, need to call errorcallback
	                    	try{
	                    		anyExp = egl.egl.jsrt.$ServiceRT.createExceptionFromJsonRPCError(response.body, serviceCallType);                    		
		                	}
		                	catch(jsonError){
		                		//the response.body is either not in a json format 
		                		//or the parsed json object isn't a JSONRPCError object
		                		//error parsing JSONRPCError
		                		anyExp = egl.createServiceInvocationException(
		                				"CRRUI3653E", 
		                				[request.uri], 
		                				egl.egl.jsrt.$ServiceRT.getExceptionSource(serviceCallType),
		                				response.status,
		                				response.statusMessage,
		                				response.body);
		                	}
	                    }
	                }
	                catch (e2) {
	                	anyExp = egl.createServiceInvocationException(
	                			"CRRUI3655E",
	                			[e2.message],
	                			egl.egl.jsrt.$ServiceRT.getExceptionSource(serviceCallType),
	                			response.status ? response.status : "",
	                			response.statusMessage ? response.statusMessage : "",
	                			response.body ? response.body : "");	                	                	
	                }
	                
                	if(anyExp != null)
                		egl.egl.jsrt.$ServiceRT.callErrorCallback(errCallbackFunc, uriString, handler, anyExp, response);                	
	            }, 
	            function(exception, response){
	            	egl.egl.jsrt.$ServiceRT.callErrorCallback(errCallbackFunc, uriString, handler, exception, response);
	            },
	            this
	        );
        }
    },
    
    "createExceptionFromJsonRPCError" : function (jsonRpcErroString, /*String*/ serviceCallType){
		//try to parse the response.body to json
        var exp = null;		
        var jsonObjE = egl.egl.core.$ServiceLib.convertFromJSON(jsonRpcErroString);                    	
        if(jsonObjE != null && jsonObjE.error != undefined){
    		var eglExpObj = jsonObjE.error.error;
    		
    		var expType = eglExpObj.name;
    		//create exception based on jsonErrObj
    		if(expType == "egl.core.ServiceInvocationException"){
        		exp = egl.createServiceInvocationException(
        				eglExpObj.messageID,
        				eglExpObj.message,
        				eglExpObj.source,
        				eglExpObj.detail1,
        				eglExpObj.detail2,
        				eglExpObj.detail3);
    		}
    		else if(expType == "egl.core.ServiceBindingException"){
    			exp = egl.createServiceBindingException(eglExpObj.messageID, eglExpObj.message);
    		}
    		else{
        		exp = egl.createServiceInvocationException(
        				eglExpObj.messageID,
        				eglExpObj.message,
        				egl.egl.jsrt.$ServiceRT.getExceptionSource(serviceCallType),
        				expType);		//put the exception type name in detail1	                    			
    		}                    			
        }
        else{
        	throw "jsonError";		
    	}
    	return exp;
    },    
    "callErrorCallback" : function (errCallbackFunc, uri, handler, exception, response){
    	try {
    		if(response)
	    		egl.egl.jsrt.$ServiceRT.setStaticRESTResponse(response);      	
	    	
	    	if(errCallbackFunc){
	    		if (egl.debugg) {
	    			egl.debugCallback(handler, errCallbackFunc, exception);
	    		}
	    		else {
	    			errCallbackFunc.call(handler, exception);
	    		}
	    	}
    	}
	    catch (e){
	    	egl.printError( egl.getRuntimeMessage("CRRUI1072E", [e.message] ), e);
	    	throw egl.createRuntimeException("CRRUI1072E", [e.message] );
	    }
	    
	    if(!errCallbackFunc || errCallbackFunc == null || errCallbackFunc == undefined)
	    {	//if no error call back function defined, just throw the exception
	    	egl.printError( egl.getRuntimeMessage("CRRUI1071E", [exception.message] ), exception);
	    	throw egl.createRuntimeException( "CRRUI1071E", [exception.message] ); 	    
    	}
    },
    
    "processCallbackArg" : function(/*Function or Array*/ callbackArg, /*Object*/ resultObj){
    	if(callbackArg instanceof Array){
    		//then the resultObj should also be an array
    		var arrayVal = new Array();
    		if(resultObj){
	    		for(var i=0; i<resultObj.length; i++){
	    			arrayVal.push(egl.egl.jsrt.$ServiceRT.processCallbackArg(callbackArg[0], resultObj[i]));
	    		}
    		}
    		else{	//array itself could be null
    			arrayVal = null;
    		}
    		return arrayVal;
    	}
    	else if(callbackArg instanceof Function){		
    		//it is a function that returns a new java script object
    		//invoke the function to get the new java script object
			var ret = callbackArg(resultObj);
			
			//ret should be an egl type corresponding java script object (i.e. egl record, egl.javascript.BigDecimal, Date)
			//check to see if it's an egl record
			if(ret != null && (ret instanceof egl.egl.jsrt.Record))  	
				ret.eze$$fromJson(resultObj, true);
			
			//it could be null type is nullable
			return ret;
    	}
    },
    
    "setStaticRESTResponse" : function(/*HttpResponse*/ response){
    	if(response){
	        egl.egl.core.$ServiceLib.callBackResponse.status = response.status;
	        egl.egl.core.$ServiceLib.callBackResponse.statusMessage = response.statusMessage;
	        egl.egl.core.$ServiceLib.callBackResponse.body = response.body;      
	        
	        //need to create a dictionary for the response header from java script object request.headers        
	        egl.egl.core.$ServiceLib.callBackResponse.headers = egl.createDictionary(true, false);    
	        egl.egl.core.$ServiceLib.convertObjectToDictionary(response.headers, egl.egl.core.$ServiceLib.callBackResponse.headers);
    	}
    },

    "doInvokeAsync" : function( /*HttpRequest*/ request, 
    							/*String */ serviceCallType,
    							/*HttpCallback*/ callback, 
    							/*HttpCallback*/ errCallback, 
    							/*handler*/ handler ) {
        this.doInvokeInternal(
        		request, 
        		serviceCallType,
        		function(response){
        			egl.startNewWork();
                    callback.call( handler, response );
                }, 
                function(exception, response){
        			egl.startNewWork();
                	errCallback.call(handler, exception, response);
                },
                true);
    },
    "doInvokeInternal" : function( /*HttpRequest*/ request, 
    							   /*String*/ serviceCallTypeIn,
                                   /*function*/ callback, 
                                   /*function*/ errCallback,                     
                                   /*boolean*/ asynchronous) {
    	var serviceCallType = serviceCallTypeIn;     //copy the value
        var requestString = egl.egl.core.$ServiceLib.convertToJSON(request);
        egl.egl.core.$ServiceLib.callBackResponse.originalRequest = requestString;
        var xhr = egl.newXMLHttpRequest();
        var beginTime = new Date().getTime();
        var response = new egl.egl.core.HttpResponse();
        var proxyURL = '/' + egl.contextRoot + '/' + this.eze$$proxyURL;
        
        if (egl.debugg || egl.contextAware) {
	        if (proxyURL.indexOf("?") == -1) {
	        	proxyURL += "?contextKey=" + egl.getContextKey();
	        }
	        else {
	        	proxyURL += "&contextKey=" + egl.getContextKey();
	        }
        }
        
        xhr.onreadystatechange = function(){
            //4 - the response is complete; you can get the server's response and use it
            if (xhr.readyState == 4) {
            	var eglExp = null;
                try {                    
                	egl.enter("Response for " + request.uri, { eze$$typename: "RUI Runtime" });
                    if (xhr.status >= 200 && xhr.status <= 299){
                        var json;
                        try {
                        	egl.enter("Parse JSON ("+xhr.responseText.length+" bytes) from "+ request.uri, { eze$$typename: "RUI Runtime" });
                        	json = egl.egl.core.$ServiceLib.convertFromJSON(xhr.responseText);
                        }
                        finally {
                        	egl.leave();
                        }
                        if (json != null) {
                            response = json;
                            response.body = response.body.replace(/&quot;/g, '"');
                            try {
                            	egl.enter("Handling callback for " + request.uri, { eze$$typename: "RUI Runtime" });
                            	egl.egl.jsrt.$ServiceRT.runCallback(callback, response, request.uri, beginTime);
                            }
                            finally {
                            	egl.leave();
                            }
                        }
                        else {
                        	response.status = xhr.status;
                        	response.statusMessage = xhr.statusMessage;
                        	eglExp = egl.createServiceInvocationException(
                            		"CRRUI3659E",
                            		[xhr.responseText],
                            		egl.egl.jsrt.$ServiceRT.getExceptionSource(serviceCallType),
                            		xhr.status,
                            		xhr.statusMessage);
                        }
                    }
                    else {
	                	//things failed on the proxy, need to report error to errorCallback                    	
                        response.status = xhr.status || 404;
                        response.statusMessage = xhr.statusText;     
                        
                    	//var exp = null;
                    	try{
                    		eglExp = egl.egl.jsrt.$ServiceRT.createExceptionFromJsonRPCError(xhr.responseText, serviceCallType);                    		
	                	}
	                	catch(jsonError){
	                		//the xhr.responseText is either not in a json format 
	                		//or the parsed json object isn't a JSONRPCError object
	                		//error parsing JSONRPCError
	                		if(response.status == 404){
	                			eglExp = egl.createServiceInvocationException(
	                					"CRRUI3657E",
	                					[proxyURL],
	                					egl.egl.jsrt.$ServiceRT.getExceptionSource(serviceCallType),
		                				response.status,
		                				response.statusMessage,
		                				xhr.responseText);
	                		}
	                		else{	
	                			eglExp = egl.createServiceInvocationException(
		                				"CRRUI3658E", 
		                				[proxyURL, request.uri],
		                				egl.egl.jsrt.$ServiceRT.getExceptionSource(serviceCallType),
		                				response.status,
		                				response.statusMessage,
		                				xhr.responseText);
	                		}
	                	}
                    }                    
                }
                catch(e) {
                	eglExp = egl.createServiceInvocationException(
                			"CRRUI3660E",
                			[request.uri, e.message],
                			egl.egl.jsrt.$ServiceRT.getExceptionSource(serviceCallType),
                			xhr.status,
                			xhr.statusMessage,
                			xhr.responseText);
                }
                finally {
                	egl.leave();
                }
                
                if(eglExp != null)
                	egl.egl.jsrt.$ServiceRT.runErrorCallback(errCallback, eglExp, response);
            }
        }
        if(egl.eze$$SetProxyAuth == true){
         	xhr.open('POST', proxyURL, asynchronous, egl.egl.core.$ServiceLib.eze$$proxyUser,  egl.egl.core.$ServiceLib.eze$$proxyPwd);
         	egl.eze$$SetProxyAuth = false;		//reset it
        }
        else
        	xhr.open( 'POST', proxyURL, asynchronous );
        
        //to use php
        //xhr.open( 'POST', '___proxy.php?url='+encodeURIComponent(request.uri), asynchronous );
        xhr.setRequestHeader('Content-Length', requestString.length);
        xhr.setRequestHeader('Content-Type', 'application/jsonrequest;charset=UTF-8');

        xhr.send( requestString );
    },
    "runCallback" : function(callback, response, uri, beginTime) {
        if (egl.debug) {
            egl.enter("<font color=darkgreen>after&nbsp;"+
                (beginTime ? (new Date().getTime()-beginTime) : 0)+
                "ms,&nbsp;handle&nbsp;service&nbsp;response&nbsp;for&nbsp;"+
                uri+"</font>", this, []);
        }
        try {
            callback( response );
        }
        finally {
            if (egl.debug) {
                egl.leave("service&nbsp;response&nbsp;for&nbsp;"+uri, this, []);
            }
        }        
    },
    "runErrorCallback" : function(errCallback, exception, response) {
    	if (errCallback) {
    		errCallback(exception, response);
    	}
    	else {
    		throw egl.createRuntimeException( "CRRUI1071E", [request.uri] );
    	}
    }
});

new egl.egl.jsrt.ServiceRT();
egl.statefulSessionMap = new Object();
egl.eze$$SetProxyAuth=null;
